package com.qmth.distributed.print.business.templete.push;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.service.TBSyncTaskService;
import com.qmth.distributed.print.business.templete.importData.AsyncImportTaskTemplete;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SpringContextHolder;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.entity.TBTask;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.enums.TaskTypeEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.service.SysConfigService;
import com.qmth.teachcloud.common.service.TBTaskService;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;

import javax.annotation.Resource;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 异步推送模板
 * @Author: CaoZixuan
 * @Date: 2021-10-31
 */
public abstract class AsyncPushTaskTemplate {
    @Resource
    private DictionaryConfig dictionaryConfig;
    @Resource
    private TBSyncTaskService tbSyncTaskService;

    private final static Logger log = LoggerFactory.getLogger(AsyncPushTaskTemplate.class);

    public static final String BEGIN_TITLE = "->开始准备处理同步的";
    public static final String EXCEPTION_TITLE = "->数据处理发生异常!";
    public static final String EXCEPTION_DATA = "错误信息:";
    public static final String FINISH_TITLE = "->数据处理结束，共需处理";
    public static final String SUCCESS_TITLE = "条数据，其中成功处理了";
    public static final String FINISH_SIZE = "条数据。";
    public static final String TXT_PREFIX = ".txt";
    public static final String EXCEPTION_CREATE_TXT_TITLE = "->创建导出日志时发生异常!";

    /**
     * 异步导入任务
     *
     * @param map
     * @return
     * @throws IOException
     */
    @Async
    public abstract Result pushTask(Map<String, Object> map) throws IOException, Exception;

    /**
     * 创建txt文件
     *
     * @param tbSyncTask
     */
    public void createTxt(TBSyncTask tbSyncTask) {
        FileStoreUtil fileStoreUtil = SpringContextHolder.getBean(FileStoreUtil.class);
        ByteArrayOutputStream out = null;
        InputStream inputStream = null;
        try {
            String charset = SystemConstant.CHARSET_GBK;
            CommonCacheService commonCacheService = SpringContextHolder.getBean(CommonCacheService.class);
            if(commonCacheService != null) {
                SysConfig sysConfig = commonCacheService.txtCharsetCache();
                if (sysConfig != null && StringUtils.isNotBlank(sysConfig.getConfigValue())) {
                    charset = sysConfig.getConfigValue();
                }
            }

            out = new ByteArrayOutputStream();
            out.write(tbSyncTask.getSummary().getBytes(charset));
            byte[] bookByteAry = out.toByteArray();
            inputStream = new ByteArrayInputStream(bookByteAry);

            boolean oss = dictionaryConfig.sysDomain().isOss();
            StringJoiner stringJoiner = new StringJoiner("");
            JSONObject json = new JSONObject();
            if (oss) {
                json.put(SystemConstant.TYPE, SystemConstant.OSS);
            } else {
                stringJoiner.add(SystemConstant.TEMP_FILES_DIR).add(File.separator);
                json.put(SystemConstant.TYPE, SystemConstant.LOCAL);
            }
            stringJoiner.add(UploadFileEnum.FILE.getTitle()).add(File.separator);
            LocalDateTime nowTime = LocalDateTime.now();
            stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                    .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                    .add(String.format("%02d", nowTime.getDayOfMonth()))
                    .add(File.separator).add(SystemConstant.getUuid()).add(TXT_PREFIX);

            String path = stringJoiner.toString().replaceAll("\\\\","/");
            String type = (String) json.get(SystemConstant.TYPE);

            //上传至oss
            if (Objects.equals(type, SystemConstant.OSS)) {
                fileStoreUtil.ossUpload(path, inputStream, DigestUtils.md5Hex(new ByteArrayInputStream(bookByteAry)), fileStoreUtil.getUploadEnumByPath(stringJoiner.toString()).getFssType());
            } else {//上传至服务器
                File finalFile = new File(stringJoiner.toString());
                if (!finalFile.exists()) {
                    finalFile.getParentFile().mkdirs();
                    finalFile.createNewFile();
                }
                FileUtils.copyInputStreamToFile(inputStream, finalFile);
            }
            json.put(SystemConstant.PATH, path);
            json.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);

            tbSyncTask.setReportFilePath(json.toJSONString());
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            StringJoiner stringJoinerSummary = new StringJoiner("").add(tbSyncTask.getSummary()).add("\n");
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", DateUtil.format(new Date(), SystemConstant.DEFAULT_DATE_PATTERN), EXCEPTION_CREATE_TXT_TITLE, EXCEPTION_DATA, e.getMessage()));
            tbSyncTask.setSummary(stringJoinerSummary.toString());
            tbSyncTask.setResult(TaskResultEnum.ERROR);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            try {
                if (Objects.nonNull(inputStream)) {
                    inputStream.close();
                }
                if (Objects.nonNull(out)) {
                    out.flush();
                    out.close();
                }
            } catch (IOException e) {
                log.error(SystemConstant.LOG_ERROR, e);
            }
            tbSyncTask.setStatus(TaskStatusEnum.FINISH);
            tbSyncTaskService.updateById(tbSyncTask);
        }
    }
}
