package com.qmth.teachcloud.common.service.impl;

import com.qmth.teachcloud.common.bean.dto.TBSyncTaskDto;
import com.qmth.teachcloud.common.bean.params.UserPushParam;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import com.qmth.teachcloud.common.mapper.CloudUserPushOperateMapper;
import com.qmth.teachcloud.common.service.CloudUserPushOperateService;
import com.qmth.teachcloud.common.service.SysUserService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 推送任务服务实现类
 * @Author: CaoZixuan
 * @Date: 2022-08-03
 */
@Service
public class CloudUserPushOperateServiceImpl implements CloudUserPushOperateService {
    @Resource
    private CloudUserPushOperateMapper cloudUserPushOperateMapper;
    @Resource
    private SysUserService sysUserService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateTBSyncTask(TBSyncTaskDto tbSyncTaskDto) {
        Long id = tbSyncTaskDto.getId();
        if (SystemConstant.longNotNull(id)) {
            // 存在id -> 如果查询数据库中已存该id的对象则更新，否则新增 || 不存在id -> 新增
            TBSyncTaskDto db = cloudUserPushOperateMapper.findById(id);
            if (Objects.nonNull(db)) {
                // 更新
                boolean updateResult = cloudUserPushOperateMapper.updateTBSyncTask(tbSyncTaskDto);
                if (!updateResult) {
                    throw ExceptionResultEnum.ERROR.exception("推送任务更新失败");
                }
            } else {
                // 新增
                boolean insertResult = cloudUserPushOperateMapper.insertTBSyncTask(tbSyncTaskDto);
                if (!insertResult) {
                    throw ExceptionResultEnum.ERROR.exception("推送任务新增失败");
                }
            }
        } else {
            id = SystemConstant.getDbUuid();
            tbSyncTaskDto.setId(id);
            boolean insertResult = cloudUserPushOperateMapper.insertTBSyncTask(tbSyncTaskDto);
            if (!insertResult) {
                throw ExceptionResultEnum.ERROR.exception("推送任务新增失败");
            }
        }
    }

    @Override
    public TBSyncTaskDto findUserPushRecord(Long userId, Long schoolId) {
        List<TBSyncTaskDto> userPushList = cloudUserPushOperateMapper.findPushRecord(schoolId, userId, PushTypeEnum.USER_PUSH.name());
        if (userPushList != null && userPushList.size() > 0) {
            if (userPushList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("异常：用户【" + userId + "】拥有多条推送记录");
            }
            return userPushList.get(0);
        } else {
            return null;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void initUserPushTasks(Map<Long, List<UserPushParam>> userPushMap, SysUser requestUser) {
        Long schoolId = requestUser.getSchoolId();
        for (Long userId : userPushMap.keySet()) {
            TBSyncTaskDto dbRecord = this.findUserPushRecord(userId, schoolId);
            Long id;
            if (Objects.nonNull(dbRecord)) {
                id = dbRecord.getId();
            } else {
                id = SystemConstant.getDbUuid();
            }
            TBSyncTaskDto tbSyncTaskDto = new TBSyncTaskDto();
            tbSyncTaskDto.setId(id);
            tbSyncTaskDto.setSchoolId(schoolId);
            tbSyncTaskDto.setType(PushTypeEnum.USER_PUSH.name());
            tbSyncTaskDto.setObjectId(userId);
            tbSyncTaskDto.setStatus(TaskStatusEnum.INIT.name());
            tbSyncTaskDto.setRemark(sysUserService.getById(userId).getLoginName());
            tbSyncTaskDto.setResult(null);
            tbSyncTaskDto.setCreateId(requestUser.getId());
            tbSyncTaskDto.setCreateTime(System.currentTimeMillis());
            this.saveOrUpdateTBSyncTask(tbSyncTaskDto);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TBSyncTaskDto runningUserPushTask(Long userId, SysUser requestUser) {
        Long schoolId = requestUser.getSchoolId();
        TBSyncTaskDto tbSyncTaskDto = this.findUserPushRecord(userId, schoolId);
        if (Objects.isNull(tbSyncTaskDto)) {
            throw ExceptionResultEnum.ERROR.exception("用户推送表信息缺失");
        }
        tbSyncTaskDto.setStatus(TaskStatusEnum.RUNNING.name());
        tbSyncTaskDto.updateInfo(requestUser.getId());
        this.saveOrUpdateTBSyncTask(tbSyncTaskDto);
        return tbSyncTaskDto;
    }
}
