SET NAMES UTF8;

-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
DROP TABLE IF EXISTS `basic_attachment`;
CREATE TABLE `basic_attachment` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                    `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                    `name` varchar(100) NOT NULL COMMENT '文件名',
                                    `type` varchar(10) NOT NULL COMMENT '文件类型',
                                    `size` int(11) NOT NULL COMMENT '单位（KB）',
                                    `md5` varchar(32) NOT NULL COMMENT '文件md5值',
                                    `path` varchar(2000) NOT NULL COMMENT '存储路径',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    `obj_id` bigint(20) DEFAULT NULL COMMENT '关联业务id',
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='所有附件记录表';

-- ----------------------------
-- Table structure for basic_campus
-- ----------------------------
DROP TABLE IF EXISTS `basic_campus`;
CREATE TABLE `basic_campus` (
                                `id` bigint(20) NOT NULL COMMENT '校区id',
                                `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                `campus_name` varchar(64) DEFAULT NULL COMMENT '校区名称',
                                `campus_code` varchar(64) DEFAULT NULL COMMENT '校区编码',
                                `print_house_id` bigint(20) DEFAULT NULL COMMENT '印刷室id',
                                `enable` tinyint(4) DEFAULT '1' COMMENT '是否可用',
                                `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `campus_schoolId_code_idx` (`school_id`,`campus_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='基础校区表';

-- ----------------------------
-- Table structure for basic_card_rule
-- ----------------------------
DROP TABLE IF EXISTS `basic_card_rule`;
CREATE TABLE `basic_card_rule` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                   `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                   `name` varchar(30) NOT NULL COMMENT '题卡规则名称',
                                   `exam_number_style` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，PASTE-粘贴条码，FILL-卡号填涂',
                                   `paper_type` varchar(5) NOT NULL COMMENT 'PRINT-印刷条码，FILL-卡号填涂',
                                   `exam_absent` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                   `write_sign` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                   `discipline` tinyint(1) DEFAULT NULL COMMENT '违纪填涂（1：是，0：否）',
                                   `required_fields` text NOT NULL COMMENT '必选字段',
                                   `extend_fields` text NOT NULL COMMENT '扩展字段',
                                   `title_rule` varchar(200) NOT NULL COMMENT '题卡标题规则',
                                   `attention` text NOT NULL COMMENT '注意事项',
                                   `objective_attention` text NOT NULL COMMENT '客观题注意事项',
                                   `subjective_attention` text NOT NULL COMMENT '主观题注意事项',
                                   `enable` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                   `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='题卡规则';

-- ----------------------------
-- Table structure for basic_clazz
-- ----------------------------
DROP TABLE IF EXISTS `basic_clazz`;
CREATE TABLE `basic_clazz` (
                               `id` bigint(20) NOT NULL,
                               `school_id` bigint(20) NOT NULL COMMENT '学校id',
                               `campus_id` bigint(20) NOT NULL COMMENT '校区id',
                               `major_id` bigint(20) NOT NULL COMMENT '专业id',
                               `clazz_code` varchar(50) DEFAULT NULL COMMENT '班级编号',
                               `clazz_name` varchar(50) DEFAULT NULL COMMENT '班级名称',
                               `enable` tinyint(1) DEFAULT NULL COMMENT '是否可用',
                               `org_id` bigint(20) DEFAULT NULL COMMENT '权限机构id',
                               `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                               PRIMARY KEY (`id`) USING BTREE,
                               UNIQUE KEY `clazz_schoolId_name_idx` (`school_id`,`clazz_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='基础班级表';

-- ----------------------------
-- Table structure for basic_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_course`;
CREATE TABLE `basic_course` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                `org_id` bigint(20) NOT NULL COMMENT '机构id',
                                `code` varchar(20) NOT NULL COMMENT '课程编码',
                                `name` varchar(50) NOT NULL COMMENT '课程名称',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                                `teaching_room_id` bigint(20) DEFAULT NULL COMMENT '教研室id',
                                `enable` tinyint(4) DEFAULT NULL,
                                `historic_name` varchar(100) DEFAULT NULL COMMENT '历史课程名称',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `course_schoolId_code_idx` (`school_id`,`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='课程表';

-- ----------------------------
-- Table structure for basic_exam
-- ----------------------------
DROP TABLE IF EXISTS `basic_exam`;
CREATE TABLE `basic_exam` (
                              `id` bigint(20) NOT NULL,
                              `school_id` bigint(20) NOT NULL,
                              `semester_id` bigint(20) DEFAULT NULL COMMENT '学期ID',
                              `name` varchar(100) DEFAULT NULL COMMENT '考试名称',
                              `category` varchar(45) DEFAULT NULL COMMENT '考试类型（ONLINE：线上，OFFLINE：线下）',
                              `exam_model` varchar(45) DEFAULT NULL COMMENT '业务模式（MODEL1：电子交卷环节需要提交考务数据，MODEL2：电子交卷环节不需要提交考务数据，只输入印刷份数即可，MODEL3：电子交卷环节不需要提交考务数据）',
                              `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
                              `in_used` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1是/0否',
                              `create_id` bigint(20) DEFAULT NULL,
                              `create_time` bigint(20) DEFAULT NULL,
                              `update_id` bigint(20) DEFAULT NULL,
                              `update_time` bigint(20) DEFAULT NULL,
                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考试表';

-- ----------------------------
-- Table structure for basic_exam_rule
-- ----------------------------
DROP TABLE IF EXISTS `basic_exam_rule`;
CREATE TABLE `basic_exam_rule` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                   `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                   `required_fields` text COMMENT '必选字段',
                                   `extend_fields` text NOT NULL COMMENT '扩展字段',
                                   `review` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                   `include_paper` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                   `custom_card` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                   `print_method` varchar(6) NOT NULL COMMENT 'AUTO-自动，MANUAL-手动',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考务规则';

-- ----------------------------
-- Table structure for basic_major
-- ----------------------------
DROP TABLE IF EXISTS `basic_major`;
CREATE TABLE `basic_major` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `school_id` bigint(20) NOT NULL COMMENT '学校id',
                               `org_id` bigint(20) NOT NULL COMMENT '机构id',
                               `name` varchar(100) NOT NULL COMMENT '专业名称',
                               `code` varchar(100) DEFAULT NULL COMMENT '专业编码',
                               `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                               `belong_org_id` bigint(20) NOT NULL COMMENT '所属机构id',
                               PRIMARY KEY (`id`) USING BTREE,
                               UNIQUE KEY `major_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='专业字典表';

-- ----------------------------
-- Table structure for basic_message
-- ----------------------------
DROP TABLE IF EXISTS `basic_message`;
CREATE TABLE `basic_message` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                 `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                 `user_id` bigint(20) NOT NULL COMMENT '消息接收人用户',
                                 `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
                                 `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
                                 `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
                                 `course_code` varchar(50) DEFAULT NULL COMMENT '课程代码',
                                 `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
                                 `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
                                 `business_id` bigint(20) DEFAULT NULL COMMENT '业务id',
                                 `template_code` varchar(45) NOT NULL COMMENT '消息模板代码',
                                 `variable_params` text COMMENT '变量参数内容',
                                 `template_content` mediumtext COMMENT '模板内容',
                                 `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
                                 `send_result` varchar(200) DEFAULT NULL COMMENT '消息发送结果',
                                 `remark` text COMMENT '备注',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                 `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                 `resend_count` int(11) DEFAULT NULL COMMENT '重试次数',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for basic_operation_log
-- ----------------------------
DROP TABLE IF EXISTS `basic_operation_log`;
CREATE TABLE `basic_operation_log` (
                                       `id` bigint(20) NOT NULL COMMENT 'id',
                                       `detail` mediumtext NOT NULL COMMENT '日志详情',
                                       `level` varchar(32) DEFAULT NULL COMMENT '日志等级（PRIMARY("一级日志：敏感信息"),SECONDARY("二级日志：重要信息"),TERTIARY("三级日志：次要信息"),QUATERNARY("四级日志：普通信息")）',
                                       `operation_type` varchar(32) DEFAULT NULL COMMENT '操作类型（UN_KNOW("未知"),SELECT("查询"),EDIT("新增或更新"),DELETE("删除")）',
                                       `operation_unit` varchar(128) DEFAULT NULL COMMENT '操作对象（略）',
                                       `system_type` varchar(32) DEFAULT NULL COMMENT '日志系统类型',
                                       `url` varchar(64) DEFAULT NULL COMMENT '请求地址url',
                                       `clazz_name` varchar(64) DEFAULT NULL COMMENT '类名称',
                                       `method` varchar(64) DEFAULT NULL COMMENT '方法名称',
                                       `args` mediumtext COMMENT '参数',
                                       `run_time` bigint(20) DEFAULT NULL COMMENT '方法运行时间',
                                       `run_status` varchar(32) DEFAULT NULL COMMENT '方法运行状态',
                                       `return_value` mediumtext COMMENT '方法返回值',
                                       `user_id` bigint(20) NOT NULL COMMENT '用户对象id',
                                       `user_name` varchar(32) DEFAULT NULL COMMENT '用户名称',
                                       `create_time` bigint(20) DEFAULT NULL COMMENT '日志记录时间',
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='基础日志记录表';

-- ----------------------------
-- Table structure for basic_print_config
-- ----------------------------
DROP TABLE IF EXISTS `basic_print_config`;
CREATE TABLE `basic_print_config` (
                                      `id` bigint(20) NOT NULL,
                                      `school_id` bigint(20) NOT NULL COMMENT '学校ID',
                                      `exam_id` bigint(20) DEFAULT NULL COMMENT '考试ID',
                                      `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '1启用/0禁用',
                                      `card_rule_id` bigint(20) DEFAULT NULL COMMENT '题卡规则（-1代表通卡）',
                                      `print_content` varchar(255) DEFAULT NULL COMMENT '试卷、题卡',
                                      `backup_method` varchar(10) DEFAULT NULL COMMENT '试卷题卡备份方式（ROOM按考场）',
                                      `backup_count` int(3) DEFAULT NULL COMMENT '试卷题卡备份数量',
                                      `draw_rule` varchar(10) DEFAULT NULL COMMENT '抽卷规则（ONE只抽一次，CIRCLE循环抽取）',
                                      `variable_content` mediumtext COMMENT '变量印品参数json',
                                      `ordinary_content` mediumtext COMMENT '普通印品参数json',
                                      `create_id` bigint(20) DEFAULT NULL,
                                      `create_time` bigint(20) DEFAULT NULL,
                                      `update_id` bigint(20) DEFAULT NULL,
                                      `update_time` bigint(20) DEFAULT NULL,
                                      PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='印品配置表';

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
DROP TABLE IF EXISTS `basic_school`;
CREATE TABLE `basic_school` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `code` varchar(20) NOT NULL COMMENT '学校编码',
                                `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
                                `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
                                `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
                                `remark` text COMMENT '备注',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                `logo` varchar(500) DEFAULT NULL COMMENT '学校logo',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `code` (`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学校表';

-- ----------------------------
-- Table structure for basic_semester
-- ----------------------------
DROP TABLE IF EXISTS `basic_semester`;
CREATE TABLE `basic_semester` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                  `name` varchar(100) NOT NULL COMMENT '学期名称',
                                  `code` varchar(100) DEFAULT NULL COMMENT '学期编码',
                                  `start_time` bigint(20) DEFAULT NULL COMMENT '开始日期',
                                  `end_time` bigint(20) DEFAULT NULL COMMENT '结束日期',
                                  `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                  `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  `in_used` tinyint(1) DEFAULT NULL COMMENT '是否当前使用学期',
                                  PRIMARY KEY (`id`) USING BTREE,
                                  UNIQUE KEY `semester_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学期字典表';

-- ----------------------------
-- Table structure for basic_student
-- ----------------------------
DROP TABLE IF EXISTS `basic_student`;
CREATE TABLE `basic_student` (
                                 `id` bigint(20) NOT NULL COMMENT '学生id',
                                 `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                 `student_name` varchar(64) NOT NULL COMMENT '学生姓名',
                                 `student_code` varchar(64) NOT NULL COMMENT '学生编号',
                                 `phone_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
                                 `campus_id` bigint(20) DEFAULT NULL COMMENT '校区主键',
                                 `belong_org_id` bigint(20) DEFAULT NULL COMMENT '所属院系',
                                 `major_id` bigint(20) DEFAULT NULL COMMENT '专业id',
                                 `clazz_id` bigint(20) DEFAULT NULL COMMENT '班级主键',
                                 `enable` tinyint(4) DEFAULT '1' COMMENT '可用状态（1：可用，0：不可用）',
                                 `org_id` bigint(20) DEFAULT NULL COMMENT '可操作人机构id',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                 `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                 PRIMARY KEY (`id`) USING BTREE,
                                 UNIQUE KEY `student_schoolId_code_orgId_idx` (`school_id`,`student_code`,`belong_org_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学校id';

-- ----------------------------
-- Table structure for basic_template
-- ----------------------------
DROP TABLE IF EXISTS `basic_template`;
CREATE TABLE `basic_template` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                  `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                  `name` varchar(30) NOT NULL COMMENT '模板名称',
                                  `type` varchar(8) NOT NULL COMMENT '模板类型：GENERIC-通卡模板，VARIABLE-变量印品模板，ORDINARY-普通印品模板',
                                  `classify` varchar(10) DEFAULT NULL COMMENT 'template_type值为VARIABLE、ORDINARY时，不为空。\n  SIGN-签到表，PACKAGE-卷袋贴，CHECK_IN-登记表',
                                  `attachment_id` bigint(20) NOT NULL COMMENT '模板附件ID（保存到附件表ID）',
                                  `enable` tinyint(4) NOT NULL COMMENT '0-禁用，1-启用',
                                  `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='通用模板';

-- ----------------------------
-- Table structure for basic_template_org
-- ----------------------------
DROP TABLE IF EXISTS `basic_template_org`;
CREATE TABLE `basic_template_org` (
                                      `id` bigint(20) NOT NULL COMMENT '主键',
                                      `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                      `type` varchar(20) NOT NULL COMMENT '模板类型：GENERIC-通卡模板，VARIABLE-变量印品模板，ORDINARY-普通印品模板，CARD_RULE-题卡规则，PRINT_CONFIG-印品配置',
                                      `rule_id` bigint(20) NOT NULL COMMENT '通卡模板ID',
                                      `org_id` bigint(20) NOT NULL COMMENT '组织机构ID',
                                      `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                      `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                      `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                      `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='适用学院（题卡规则、模板适用学院）';

-- ----------------------------
-- Table structure for basic_user_course
-- ----------------------------
DROP TABLE IF EXISTS `basic_user_course`;
CREATE TABLE `basic_user_course` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                     `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint(20) DEFAULT NULL COMMENT '用户ID',
                                     `course_id` bigint(20) DEFAULT NULL COMMENT '课程ID',
                                     `clazz_id` bigint(20) DEFAULT NULL COMMENT '授课班级',
                                     `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用？1-是 ；0-否',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='用户和课程关联表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
DROP TABLE IF EXISTS `basic_verify_code`;
CREATE TABLE `basic_verify_code` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                     `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint(20) NOT NULL COMMENT '用户ID',
                                     `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
                                     `expire_time` bigint(20) NOT NULL COMMENT '到期时间',
                                     `valid_period` int(11) NOT NULL COMMENT '单位（分）',
                                     `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for client_print_data
-- ----------------------------
DROP TABLE IF EXISTS `client_print_data`;
CREATE TABLE `client_print_data` (
                                     `id` bigint(20) NOT NULL COMMENT '主键',
                                     `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                     `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                     `exam_detail_id` bigint(20) DEFAULT NULL COMMENT '考场ID',
                                     `machine_code` varchar(45) DEFAULT NULL COMMENT '机器唯一码',
                                     `pages_a4` int(11) DEFAULT NULL COMMENT 'A4页数量',
                                     `pages_a3` int(11) DEFAULT NULL COMMENT 'A3页数量',
                                     `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='各机器打印数量';

-- ----------------------------
-- Table structure for client_status
-- ----------------------------
DROP TABLE IF EXISTS `client_status`;
CREATE TABLE `client_status` (
                                 `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '主键',
                                 `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                 `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                 `machine_code` varchar(100) COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '机器唯一码',
                                 `exam_detail_id` bigint(20) DEFAULT NULL COMMENT '考场ID',
                                 `print_plan_id` bigint(20) DEFAULT NULL COMMENT '印刷计划ID',
                                 `course_code` varchar(100) COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '课程代码',
                                 `course_name` varchar(200) COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '课程名称',
                                 `paper_number` varchar(50) COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '试卷编号',
                                 `is_try` tinyint(4) DEFAULT NULL COMMENT '是否打样：true-是，false-否',
                                 `try_time` bigint(20) DEFAULT NULL COMMENT '打样时间',
                                 `is_pass` tinyint(4) DEFAULT NULL COMMENT '是否合格：true-合格，false-不合格',
                                 `is_download` tinyint(4) DEFAULT NULL COMMENT '是否缓存：true-是，false-否',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                 `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB AUTO_INCREMENT=281784714686103553 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT='机器打样记录';

-- ----------------------------
-- Table structure for exam_card
-- ----------------------------
DROP TABLE IF EXISTS `exam_card`;
CREATE TABLE `exam_card` (
                             `id` bigint(20) NOT NULL COMMENT '主键',
                             `school_id` bigint(20) NOT NULL COMMENT '学校id',
                             `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                             `course_code` varchar(30) DEFAULT NULL COMMENT '课程编码',
                             `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                             `title` varchar(200) NOT NULL COMMENT '标题',
                             `make_method` varchar(10) DEFAULT NULL COMMENT '题卡制作方式：SELECT-选择已有题卡，SELF-自助创建，CUST-客户制卡',
                             `status` varchar(45) DEFAULT NULL COMMENT 'STAGE-暂存，SUBMIT-提交',
                             `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                             `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                             `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                             `type` varchar(45) NOT NULL COMMENT '题卡类型：GENERIC-通卡，CUSTOM-自定义',
                             `create_method` varchar(45) DEFAULT NULL COMMENT '通卡创建方式（UPLOAD-上传，CUSTOMIZE-自定义）',
                             `attachment_id` bigint(20) DEFAULT NULL COMMENT 'type=GENERIC时必传，上传附件ID',
                             `sync_status` tinyint(1) DEFAULT NULL COMMENT '是否同步成功：true-是， false-否',
                             `enable` tinyint(1) DEFAULT '1' COMMENT '1正常，0禁用',
                             `remark` mediumtext COMMENT '备注',
                             `card_rule_id` bigint(20) DEFAULT NULL COMMENT '题卡规则Id',
                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='题卡';

-- ----------------------------
-- Table structure for exam_card_detail
-- ----------------------------
DROP TABLE IF EXISTS `exam_card_detail`;
CREATE TABLE `exam_card_detail` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `card_id` bigint(20) NOT NULL COMMENT '题卡ID',
                                    `content` mediumtext COMMENT '题卡工具制作题卡内容',
                                    `html_content` mediumtext COMMENT 'html格式内容',
                                    `attachment_id` mediumtext COMMENT '备用题卡json',
                                    `cust_attachment_id` bigint(20) DEFAULT NULL COMMENT '客服制卡上传附件ID',
                                    `jpg_attachment_info` text COMMENT '题卡转换成的jpg文件信息',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='题卡详情';

-- ----------------------------
-- Table structure for exam_detail
-- ----------------------------
DROP TABLE IF EXISTS `exam_detail`;
CREATE TABLE `exam_detail` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `school_id` bigint(20) NOT NULL COMMENT '学校id',
                               `org_id` bigint(20) NOT NULL COMMENT '机构id',
                               `print_plan_id` bigint(20) NOT NULL COMMENT '印刷计划ID',
                               `print_plan_name` varchar(100) DEFAULT NULL COMMENT '印刷计划名称',
                               `exam_start_time` bigint(20) NOT NULL COMMENT '考试开始日期',
                               `exam_end_time` bigint(20) NOT NULL COMMENT '考试结束日期',
                               `exam_place` varchar(30) NOT NULL COMMENT '考场',
                               `total_subjects` int(11) DEFAULT NULL COMMENT '总科次：考场总科次',
                               `exam_room` varchar(30) DEFAULT NULL COMMENT '教室',
                               `print_progress` double DEFAULT NULL COMMENT '打印进度',
                               `print_user` varchar(30) DEFAULT NULL COMMENT '打印员：一个考场只能被一个打印员认领',
                               `package_code` varchar(30) DEFAULT NULL COMMENT '考场代码：卷袋贴、签到表上条码',
                               `pages_a3` int(11) DEFAULT NULL COMMENT 'A3印量小计（页）',
                               `pages_a4` int(11) DEFAULT NULL COMMENT 'A4印量小计（页）',
                               `status` varchar(30) DEFAULT NULL COMMENT '状态：NEW: "新建",READY: "就绪",WAITING: "待印刷",PRINTING: "印刷中",FINISH: "已完成"',
                               `validate` tinyint(1) DEFAULT '0' COMMENT '是否校验：true-是，false-否',
                               `print_start_time` bigint(20) DEFAULT NULL COMMENT '打印开始时间',
                               `print_end_time` bigint(20) DEFAULT NULL COMMENT '打印完成时间',
                               `attachment_id` bigint(20) DEFAULT NULL COMMENT '附件id',
                               `card_attachment_id` bigint(20) DEFAULT NULL COMMENT '题卡合并A3文件附件ID',
                               `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                               `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                               `attachment_path` mediumtext COMMENT '印品附件路径',
                               `exam_data_source` varchar(20) DEFAULT NULL COMMENT '数据来源（EXAM_TASK：卷库查询生成，FILE_IMPORT：考务文件导入）',
                               `normal` tinyint(4) DEFAULT '1' COMMENT '是否正常，true正常，false做废',
                               `print_house_id` bigint(20) DEFAULT NULL COMMENT '印刷室ID',
                               `print_count` int(5) DEFAULT NULL COMMENT '模式2，考生数量（打印数量）',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考务数据';

-- ----------------------------
-- Table structure for exam_detail_course
-- ----------------------------
DROP TABLE IF EXISTS `exam_detail_course`;
CREATE TABLE `exam_detail_course` (
                                      `id` bigint(20) NOT NULL COMMENT '主键',
                                      `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                      `org_id` bigint(20) NOT NULL COMMENT '机构id',
                                      `exam_detail_id` bigint(20) NOT NULL COMMENT '考试明细id',
                                      `course_code` varchar(20) NOT NULL COMMENT '课程编码',
                                      `course_name` varchar(50) NOT NULL COMMENT '课程名称',
                                      `paper_number` varchar(30) NOT NULL COMMENT '页数',
                                      `total_subjects` int(11) DEFAULT NULL COMMENT '总科次：考场总科次',
                                      `paper_pages_a3` int(11) DEFAULT NULL COMMENT '试卷单科次准印量A3（页）',
                                      `card_pages_a3` int(11) DEFAULT NULL COMMENT '题卡单科次准印量A3（页）',
                                      `clazz_id` text,
                                      `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                      `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                      `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                      `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                      `paper_type` varchar(30) DEFAULT NULL COMMENT '当前试卷类型',
                                      `attachment_id` mediumtext COMMENT '备用题卡json',
                                      `common_attachment_id` mediumtext COMMENT '模式2下考生公共题卡json',
                                      PRIMARY KEY (`id`) USING BTREE,
                                      KEY `detail_course_schoolId_idx` (`school_id`,`paper_number`,`paper_type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考务数据（考场关联科目）';

-- ----------------------------
-- Table structure for exam_paper_group
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_group`;
CREATE TABLE `exam_paper_group` (
                                    `id` bigint(20) NOT NULL COMMENT '分组id',
                                    `exam_paper_structure_id` bigint(20) NOT NULL COMMENT '试卷结构id',
                                    `group_number` int(11) NOT NULL COMMENT '分组序号',
                                    `double_rate` tinyint(4) DEFAULT NULL COMMENT '双评比例（0关闭，1开启）',
                                    `arbitrate_threshold` int(11) DEFAULT NULL COMMENT '仲裁阈值（开启双评必填）',
                                    `score_policy` varchar(45) DEFAULT NULL COMMENT '合分策略 开启双评是必填 AVG("平均分", 1), MAX("最高分", 2), MIN("最低分", 3)',
                                    `trial_count` int(11) DEFAULT NULL COMMENT '试评数量',
                                    `mark_mode` varchar(45) DEFAULT NULL COMMENT '评卷模式 COMMON("普通"), TRACK("轨迹")',
                                    `question_info` text COMMENT '题目信息',
                                    `picture_config` text COMMENT '图片配置',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`),
                                    UNIQUE KEY `exam_paper_group_unique` (`exam_paper_structure_id`,`group_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='试卷分组表';

-- ----------------------------
-- Table structure for exam_paper_group_marker
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_group_marker`;
CREATE TABLE `exam_paper_group_marker` (
                                           `id` bigint(20) NOT NULL COMMENT '评卷员-分组关系id',
                                           `group_id` bigint(20) NOT NULL COMMENT '分组id',
                                           `marker_id` bigint(20) NOT NULL COMMENT '评卷员id',
                                           `login_name` varchar(100) DEFAULT NULL COMMENT '评卷员登录名',
                                           `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                           `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                           `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                           `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='评卷员-分组关系表';

-- ----------------------------
-- Table structure for exam_paper_structure
-- ----------------------------
DROP TABLE IF EXISTS `exam_paper_structure`;
CREATE TABLE `exam_paper_structure` (
                                        `id` bigint(20) NOT NULL COMMENT '主键',
                                        `school_id` bigint(20) DEFAULT NULL COMMENT '学校ID',
                                        `semester_id` bigint(20) DEFAULT NULL COMMENT '学期ID',
                                        `semester_name` varchar(100) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '学期',
                                        `exam_id` bigint(20) NOT NULL COMMENT '考试id',
                                        `third_relate_id` bigint(20) DEFAULT NULL COMMENT '云阅卷考试ID',
                                        `third_relate_name` varchar(255) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '云阅卷考试名称',
                                        `paper_number` varchar(50) COLLATE utf8mb4_unicode_ci NOT NULL COMMENT '试卷编号',
                                        `course_code` varchar(50) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '课程代码',
                                        `course_name` varchar(255) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '课程名称',
                                        `sequence` varchar(5) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '课程创建的任务序号（和exam_task保持一致）',
                                        `paper_type` varchar(20) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '试卷类型',
                                        `status` varchar(20) COLLATE utf8mb4_unicode_ci DEFAULT NULL COMMENT '状态。INIT：未开始；START：开始同步；PAPER_FINISH：试卷同步成功；ANSWER_FINISH：标答同步成功；OBJECTIVE_FINISH：客观题结构同步成功；SUBJECTIVE_FINISH：主观题结构同步成功；FINISH：结束。',
                                        `paper_answer` text COLLATE utf8mb4_unicode_ci COMMENT '试卷原卷和标答附件ID',
                                        `objective_structure` text COLLATE utf8mb4_unicode_ci COMMENT '客观题试卷结构JSON',
                                        `subjective_structure` text COLLATE utf8mb4_unicode_ci COMMENT '主观题试卷结构JSON',
                                        `paper_info_json` text COLLATE utf8mb4_unicode_ci COMMENT '整个试卷信息json（前端提供的全部信息）',
                                        `proposition_teacher_id` bigint(20) DEFAULT NULL COMMENT '命题老师ID',
                                        `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用（0-禁用，1-启用）',
                                        `mark_leader` text COLLATE utf8mb4_unicode_ci COMMENT '科组长，样例：[{markerId:1, loginName:"mt01"}]',
                                        `cloud_info_json` text COLLATE utf8mb4_unicode_ci COMMENT '云阅卷试卷结构',
                                        `structure_change` tinyint(1) DEFAULT NULL COMMENT '云阅卷试卷结构和知学知考结构不一样（默认false）',
                                        `create_id` bigint(20) DEFAULT NULL,
                                        `create_time` bigint(20) DEFAULT NULL,
                                        `update_id` bigint(20) DEFAULT NULL,
                                        `update_time` bigint(20) DEFAULT NULL,
                                        PRIMARY KEY (`id`) USING BTREE,
                                        UNIQUE KEY `exam_paper_structure_unique` (`school_id`,`exam_id`,`paper_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;

-- ----------------------------
-- Table structure for exam_print_plan
-- ----------------------------
DROP TABLE IF EXISTS `exam_print_plan`;
CREATE TABLE `exam_print_plan` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `exam_id` bigint(20) NOT NULL COMMENT '考试id',
                                   `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                   `org_id` bigint(20) NOT NULL COMMENT '机构id',
                                   `name` varchar(150) DEFAULT NULL COMMENT '名称',
                                   `exam_start_time` bigint(20) NOT NULL COMMENT '考试开始时间',
                                   `exam_end_time` bigint(20) NOT NULL COMMENT '考试结束时间',
                                   `print_content` varchar(30) NOT NULL COMMENT '{“PAPER”, "CARD"}',
                                   `backup_method` varchar(10) NOT NULL COMMENT 'PLACE-按考点，ROOM-按考场',
                                   `backup_count` int(11) NOT NULL COMMENT '备份数量',
                                   `draw_rule` varchar(10) DEFAULT NULL COMMENT 'ONE-只抽一次，CIRCLE-可循环抽取',
                                   `variable_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "SIGN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	},\n  	{\n  		"type": "PACKAG",\n  		"usedId": 3,\n  		"backupMethod": "STUDENT",\n  		"backupCount": 5\n  	}\n  ]',
                                   `ordinary_content` text NOT NULL COMMENT '[\n  	{\n  		"type": "CHECK_IN",\n  		"usedId": 1,\n  		"backupMethod": "ROOM",\n  		"backupCount": 3\n  	}\n  ]',
                                   `status` varchar(20) NOT NULL COMMENT '印刷计划状态',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   `third_relate_id` bigint(20) DEFAULT NULL COMMENT '第三方系统关联ID（目前只有云阅卷）',
                                   `third_relate_name` varchar(100) DEFAULT NULL COMMENT '第三方系统关联考试名称（目前只有云阅卷）',
                                   `sync_status` varchar(20) DEFAULT NULL COMMENT '是否同步成功',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='印刷计划管理';

-- ----------------------------
-- Table structure for exam_student
-- ----------------------------
DROP TABLE IF EXISTS `exam_student`;
CREATE TABLE `exam_student` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                `org_id` bigint(20) NOT NULL COMMENT '机构id',
                                `exam_detail_course_id` bigint(20) NOT NULL COMMENT '考试明细课程id',
                                `student_name` varchar(30) NOT NULL COMMENT '考生姓名',
                                `student_code` varchar(30) NOT NULL COMMENT '考生代码',
                                `ticket_number` varchar(30) DEFAULT NULL COMMENT '准考证号',
                                `site_number` varchar(30) DEFAULT NULL COMMENT '座位号',
                                `print_paper` tinyint(4) DEFAULT NULL COMMENT '打印试卷',
                                `print_card` tinyint(4) DEFAULT NULL COMMENT '打印题卡',
                                `extend_fields` text COMMENT '扩展字段',
                                `paper_type` varchar(2) DEFAULT NULL COMMENT '试卷类型',
                                `clazz_id` varchar(100) DEFAULT NULL COMMENT '班级Id',
                                `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
                                `campus_name` varchar(200) DEFAULT NULL COMMENT '校区',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                `attachment_id` bigint(20) DEFAULT NULL COMMENT '附件id',
                                `sync_status` tinyint(1) DEFAULT NULL COMMENT '是否同步成功：true-是， false-否',
                                PRIMARY KEY (`id`) USING BTREE,
                                KEY `exam_student_schoolId_idx` (`school_id`,`student_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考生';

-- ----------------------------
-- Table structure for exam_task
-- ----------------------------
DROP TABLE IF EXISTS `exam_task`;
CREATE TABLE `exam_task` (
                             `id` bigint(20) NOT NULL COMMENT '主键',
                             `exam_id` bigint(20) NOT NULL COMMENT '考试id',
                             `school_id` bigint(20) NOT NULL COMMENT '学校id',
                             `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                             `course_code` varchar(20) NOT NULL COMMENT '课程代码',
                             `course_name` varchar(50) NOT NULL COMMENT '课程名称',
                             `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
                             `paper_number` varchar(30) NOT NULL COMMENT '试卷编号',
                             `paper_number_id` bigint(20) DEFAULT NULL COMMENT '试卷编号ID（用于推送云阅卷）',
                             `sequence` varchar(5) DEFAULT NULL COMMENT '课程创建的任务序号',
                             `start_time` bigint(20) DEFAULT NULL COMMENT '命题开始时间',
                             `end_time` bigint(20) DEFAULT NULL COMMENT '命题结束时间',
                             `card_rule_id` bigint(20) NOT NULL COMMENT '题卡规则',
                             `user_id` bigint(20) DEFAULT NULL COMMENT '命题老师',
                             `status` varchar(50) DEFAULT NULL COMMENT '任务状态，DRAFT：新建，SUBMIT：已提交，STAGE：待命题',
                             `review` tinyint(4) DEFAULT NULL COMMENT '是否审核，0-禁用，1-启用',
                             `review_status` varchar(8) DEFAULT NULL COMMENT '审核状态：PASS-审核通过，NOT_PASS-审核不通过，默认为空-未审核',
                             `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                             `batch_no` varchar(32) DEFAULT NULL COMMENT '批次id',
                             `teaching_room_id` bigint(20) DEFAULT NULL COMMENT '教学单位、教研室',
                             `teacher_name` varchar(100) DEFAULT NULL COMMENT '拟卷教师',
                             `lecturer_name` varchar(100) DEFAULT NULL COMMENT '主讲教师',
                             `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
                             `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                             `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                             `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                             `flow_id` bigint(20) DEFAULT NULL COMMENT '流程id',
                             `approve_form_attachment_id` bigint(20) DEFAULT NULL COMMENT '命题任务审批表附件id',
                             PRIMARY KEY (`id`) USING BTREE,
                             KEY `task_schoolId_flowId_idx` (`school_id`,`flow_id`,`card_rule_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='命题任务';

-- ----------------------------
-- Table structure for exam_task_apply_temp
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_apply_temp`;
CREATE TABLE `exam_task_apply_temp` (
                                        `id` bigint(20) unsigned NOT NULL,
                                        `school_id` bigint(20) NOT NULL COMMENT '学校Id',
                                        `org_id` bigint(20) NOT NULL COMMENT '机构id',
                                        `exam_task_content` mediumtext COMMENT '命题任务json',
                                        `create_id` bigint(20) DEFAULT NULL,
                                        `create_time` bigint(20) DEFAULT NULL,
                                        `update_id` bigint(20) DEFAULT NULL,
                                        `update_time` bigint(20) DEFAULT NULL,
                                        PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='卷库查询-命题任务申请临时表';

-- ----------------------------
-- Table structure for exam_task_detail
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_detail`;
CREATE TABLE `exam_task_detail` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `exam_task_id` bigint(20) NOT NULL COMMENT '命题任务ID',
                                    `paper_type` varchar(20) DEFAULT NULL COMMENT '试卷类型，多个以”/“分隔，如A/B/C/D',
                                    `draw_count` int(11) DEFAULT NULL COMMENT '单次抽卷数量',
                                    `paper_attachment_ids` text COMMENT '转换为A3格式的试卷附件\n[\\n  	{\\n  		"type": "A",\\n  		"attachmentId": 12,\\n  		"totalPages": 3\\n  	},\\n  	{\\n  		"type": "B",\\n  		"attachmentId": 13,\\n  		"totalPages": 2\\n  	}\\n  ]',
                                    `card_id` bigint(20) DEFAULT NULL COMMENT '题卡ID',
                                    `make_method` varchar(45) DEFAULT NULL COMMENT '题卡绑定方式',
                                    `paper_confirm_attachment_ids` text COMMENT '[\n  	{\n  		"attachmentId": 12\n  	},\n  	{\n  		"attachmentId": 13\n  	}\n  ]',
                                    `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                                    `exposed_paper_type` varchar(20) DEFAULT NULL COMMENT '曝光卷型：已打印卷型，多个以”/“分隔，如A/B/C/D',
                                    `relate_paper_type` varchar(5) DEFAULT NULL COMMENT '已曝光卷型',
                                    `unexposed_paper_type` varchar(20) DEFAULT NULL COMMENT '未曝光卷型',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    `enable` tinyint(4) DEFAULT '1' COMMENT '0-禁用，1-启用',
                                    PRIMARY KEY (`id`) USING BTREE,
                                    KEY `exam_task_detail_idx` (`exam_task_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='命题任务明细表（入库申请）';

-- ----------------------------
-- Table structure for exam_task_paper_log
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_paper_log`;
CREATE TABLE `exam_task_paper_log` (
                                       `id` bigint(20) NOT NULL COMMENT '主键',
                                       `exam_task_id` bigint(20) NOT NULL COMMENT '命题任务ID',
                                       `paper_type` varchar(45) DEFAULT NULL COMMENT '试卷类型',
                                       `paper_attachment_ids` mediumtext COMMENT '修改后试卷',
                                       `card_id` bigint(20) DEFAULT NULL COMMENT '修改后题卡ID',
                                       `review` tinyint(1) DEFAULT NULL COMMENT '是否审核：true-审核，false-未审核',
                                       `review_status` varchar(45) DEFAULT NULL COMMENT '审核状态：PASS-通过，NOT_PASS-不通过',
                                       `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                       `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                       `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                       `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                       PRIMARY KEY (`id`) USING BTREE,
                                       KEY `exam_task_paper_idx` (`exam_task_id`,`review`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='卷库修改审核临时表';

-- ----------------------------
-- Table structure for exam_task_print
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_print`;
CREATE TABLE `exam_task_print` (
                                   `id` bigint(20) NOT NULL,
                                   `school_id` bigint(20) DEFAULT NULL,
                                   `org_id` bigint(20) DEFAULT NULL,
                                   `print_plan_id` bigint(20) DEFAULT NULL COMMENT '印刷计划ID',
                                   `course_code` varchar(45) DEFAULT NULL COMMENT '课程代码',
                                   `course_name` varchar(50) DEFAULT NULL COMMENT '课程名称',
                                   `paper_number` varchar(45) DEFAULT NULL COMMENT '试卷编号',
                                   `exam_start_time` bigint(20) DEFAULT NULL COMMENT '考试开始时间',
                                   `exam_end_time` bigint(20) DEFAULT NULL COMMENT '考试结束时间',
                                   `package_seq` varchar(30) DEFAULT NULL COMMENT '卷袋序号',
                                   `exam_place` varchar(100) DEFAULT NULL,
                                   `exam_room` varchar(100) DEFAULT NULL,
                                   `invigilator_teacher` varchar(45) DEFAULT NULL COMMENT '监考老师',
                                   `class_id` varchar(100) DEFAULT NULL COMMENT '班级ID集合',
                                   `class_name` varchar(500) DEFAULT NULL COMMENT '班级名称集合',
                                   `student_count` int(11) DEFAULT NULL COMMENT '考生总人数',
                                   `campus_id` bigint(20) DEFAULT NULL COMMENT '校区ID',
                                   `print_house_id` bigint(20) DEFAULT NULL COMMENT '印刷室ID',
                                   `print_house_name` varchar(100) DEFAULT NULL COMMENT '印刷室名称',
                                   `exam_detail_id` bigint(20) DEFAULT NULL COMMENT '考场ID',
                                   `extend_fields` text COMMENT '扩展字段',
                                   `create_id` bigint(20) DEFAULT NULL,
                                   `create_time` bigint(20) DEFAULT NULL,
                                   `update_id` bigint(20) DEFAULT NULL,
                                   `update_time` bigint(20) DEFAULT NULL,
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='以试卷作为起点生成考务数据';

-- ----------------------------
-- Table structure for exam_task_review_log
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_review_log`;
CREATE TABLE `exam_task_review_log` (
                                        `id` bigint(20) NOT NULL COMMENT '主键',
                                        `exam_task_id` bigint(20) NOT NULL COMMENT '命题任务ID',
                                        `review_status` varchar(8) NOT NULL COMMENT 'PASS-通过，NOT_PASS-不通过',
                                        `reason` mediumtext COMMENT '审核意见',
                                        `operate_id` bigint(20) DEFAULT NULL COMMENT '操作人id',
                                        `operate_time` bigint(20) DEFAULT NULL COMMENT '操作时间',
                                        `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                        `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                        `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                        `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                        PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='命题任务审核日志表';

-- ----------------------------
-- Table structure for exam_task_temp
-- ----------------------------
DROP TABLE IF EXISTS `exam_task_temp`;
CREATE TABLE `exam_task_temp` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                  `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                  `batch_no` varchar(32) NOT NULL COMMENT '本次导入数据临时ID',
                                  `course_code` varchar(20) NOT NULL COMMENT '课程代码',
                                  `course_name` varchar(50) NOT NULL COMMENT '课程名称',
                                  `paper_number` varchar(30) DEFAULT NULL COMMENT '试卷编号',
                                  `specialty` varchar(100) DEFAULT NULL COMMENT '适用专业',
                                  `user_account` varchar(45) DEFAULT NULL COMMENT '命题老师账号',
                                  `user_name` varchar(45) DEFAULT NULL COMMENT '命题老师姓名',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='命题任务临时表';

-- ----------------------------
-- Table structure for grade_batch
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch`;
CREATE TABLE `grade_batch` (
                               `id` bigint(20) NOT NULL COMMENT '分析批次主键',
                               `school_id` bigint(20) NOT NULL COMMENT '学校id',
                               `semester_id` bigint(20) NOT NULL COMMENT '学期id',
                               `batch_code` varchar(32) NOT NULL COMMENT '批次编号（在学校下唯一）',
                               `batch_name` varchar(32) NOT NULL COMMENT '批次名称（2020~2021学年上学期期末考试）',
                               `batch_time` bigint(20) NOT NULL COMMENT '分析批次时间',
                               `status` varchar(32) NOT NULL COMMENT '批次状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
                               `third_semester_id` bigint(20) DEFAULT NULL COMMENT '教研分析学期id',
                               `third_exam_id` bigint(20) DEFAULT NULL COMMENT '第三方考试id',
                               `create_id` bigint(20) DEFAULT NULL,
                               `create_time` bigint(20) DEFAULT NULL,
                               `update_id` bigint(20) DEFAULT NULL,
                               `update_time` bigint(20) DEFAULT NULL,
                               `source` varchar(1000) DEFAULT NULL COMMENT '原始数据',
                               `progress` decimal(8,2) DEFAULT NULL COMMENT '进度',
                               `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                               `notify_time` bigint(20) DEFAULT NULL COMMENT '回调时间',
                               `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='分析-批次表';

-- ----------------------------
-- Table structure for grade_batch_paper
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_paper`;
CREATE TABLE `grade_batch_paper` (
                                     `id` bigint(20) NOT NULL,
                                     `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                     `batch_id` bigint(20) NOT NULL COMMENT '分析批次id',
                                     `grade_paper_id` bigint(20) DEFAULT NULL COMMENT '试卷id',
                                     `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
                                     `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
                                     `status` varchar(32) NOT NULL COMMENT '分析试卷状态\nSETTING_GRADE_PAPER("设置分析试卷"),PUSH_GRADE_BATCH("推送教研分析批次数据"),SETTING_GRADE_PAPER_PARAM("设置分析试卷参数"),READY_TO_CALCULATE("待计算"),FINISH_CALCULATE("完成计算")',
                                     `teach_college_id` bigint(20) NOT NULL COMMENT '当前开课学院id',
                                     `teach_college_name` varchar(100) NOT NULL COMMENT '当前开课学院名称',
                                     `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
                                     `enable` tinyint(4) DEFAULT NULL COMMENT '是否可用',
                                     `publish_status` tinyint(4) DEFAULT NULL COMMENT '是否发布',
                                     `grade_course_code` varchar(100) DEFAULT NULL COMMENT '推送教研分析后回传的教研分析课程编号（考试下唯一）',
                                     `create_id` bigint(20) DEFAULT NULL,
                                     `create_time` bigint(20) DEFAULT NULL,
                                     `update_id` bigint(20) DEFAULT NULL,
                                     `update_time` bigint(20) DEFAULT NULL,
                                     PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='分析-批次试卷关联关系表';

-- ----------------------------
-- Table structure for grade_batch_student
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_student`;
CREATE TABLE `grade_batch_student` (
                                       `id` bigint(20) NOT NULL,
                                       `batch_id` bigint(20) DEFAULT NULL COMMENT '分析-批次表ID',
                                       `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
                                       `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                       `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
                                       `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
                                       `student_code` varchar(45) DEFAULT NULL COMMENT '学号',
                                       `student_name` varchar(100) DEFAULT NULL COMMENT '学生姓名',
                                       `ticket_number` varchar(45) DEFAULT NULL COMMENT '准考证号',
                                       `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
                                       `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
                                       `create_id` bigint(20) DEFAULT NULL,
                                       `create_time` bigint(20) DEFAULT NULL,
                                       `update_id` bigint(20) DEFAULT NULL,
                                       `update_time` bigint(20) DEFAULT NULL,
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='考务数据（包含任课老师）';

-- ----------------------------
-- Table structure for grade_batch_student_clazz
-- ----------------------------
DROP TABLE IF EXISTS `grade_batch_student_clazz`;
CREATE TABLE `grade_batch_student_clazz` (
                                             `id` bigint(20) NOT NULL,
                                             `batch_id` bigint(20) DEFAULT NULL COMMENT '分析-批次表ID',
                                             `course_code` varchar(50) DEFAULT NULL COMMENT '课程编码',
                                             `course_name` varchar(100) DEFAULT NULL COMMENT '课程名称',
                                             `paper_number` varchar(50) DEFAULT NULL COMMENT '试卷编号',
                                             `paper_type` varchar(16) DEFAULT NULL COMMENT '试卷类型',
                                             `clazz_id` varchar(45) DEFAULT NULL COMMENT '班级ID',
                                             `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级名称',
                                             `teacher_name` varchar(100) DEFAULT NULL COMMENT '任课老师',
                                             `teacher_number` varchar(45) DEFAULT NULL COMMENT '任课老师工号',
                                             `create_id` bigint(20) DEFAULT NULL,
                                             `create_time` bigint(20) DEFAULT NULL,
                                             `update_id` bigint(20) DEFAULT NULL,
                                             `update_time` bigint(20) DEFAULT NULL,
                                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='考务数据班级（包含任课老师）';

-- ----------------------------
-- Table structure for grade_module_define
-- ----------------------------
DROP TABLE IF EXISTS `grade_module_define`;
CREATE TABLE `grade_module_define` (
                                       `id` bigint(20) NOT NULL COMMENT '模块熟练度主键',
                                       `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                       `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
                                       `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
                                       `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
                                       `module_type` varchar(32) NOT NULL COMMENT '维度模块类型',
                                       `interpret` varchar(255) DEFAULT NULL COMMENT '解释',
                                       `remark` varchar(255) DEFAULT NULL COMMENT '备注',
                                       `define` varchar(255) DEFAULT NULL COMMENT '二级熟练度定义',
                                       `level` varchar(32) DEFAULT NULL COMMENT '等级',
                                       `min` varchar(32) DEFAULT NULL COMMENT '展示用最小边界',
                                       `max` varchar(32) DEFAULT NULL COMMENT '展示用最大边界',
                                       `scope` varchar(32) DEFAULT NULL COMMENT '取值范围（计算用）',
                                       `create_id` bigint(20) DEFAULT NULL,
                                       `create_time` bigint(20) DEFAULT NULL,
                                       `update_id` bigint(20) DEFAULT NULL,
                                       `update_time` bigint(20) DEFAULT NULL,
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='分析-模块二级考察点维度熟练度配置，及模块简要描述表';

-- ----------------------------
-- Table structure for grade_module_evaluation
-- ----------------------------
DROP TABLE IF EXISTS `grade_module_evaluation`;
CREATE TABLE `grade_module_evaluation` (
                                           `id` bigint(20) NOT NULL,
                                           `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                           `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
                                           `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
                                           `paper_name` varchar(100) DEFAULT NULL COMMENT '试卷名称',
                                           `module_type` varchar(64) NOT NULL COMMENT '模块类型（知识、维度、素养）',
                                           `formula` varchar(64) NOT NULL COMMENT '计算公式规则',
                                           `scope` varchar(64) NOT NULL COMMENT '取值范围（计算用）',
                                           `level_code` varchar(32) NOT NULL COMMENT '等级',
                                           `level_name` varchar(32) NOT NULL COMMENT '水平等级层次',
                                           `result` text COMMENT '诊断结果',
                                           `advice` text COMMENT '学习建议',
                                           `create_id` bigint(20) DEFAULT NULL,
                                           `create_time` bigint(20) DEFAULT NULL,
                                           `update_id` bigint(20) DEFAULT NULL,
                                           `update_time` bigint(20) DEFAULT NULL,
                                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='校验-考察点模块评价建议表';

-- ----------------------------
-- Table structure for grade_paper
-- ----------------------------
DROP TABLE IF EXISTS `grade_paper`;
CREATE TABLE `grade_paper` (
                               `id` bigint(20) NOT NULL COMMENT '试卷id',
                               `school_id` bigint(20) NOT NULL COMMENT '学校id',
                               `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
                               `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
                               `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
                               `start_time` bigint(20) DEFAULT NULL COMMENT '该试卷考试开始时间',
                               `end_time` bigint(20) DEFAULT NULL COMMENT '该试卷考试结束时间',
                               `coefficient` decimal(12,4) DEFAULT NULL COMMENT '赋分系数（赋值分=卷面成绩 +（卷面满分-卷面成绩）/ 赋分系数）',
                               `total_score` decimal(12,4) NOT NULL COMMENT '试卷满分',
                               `pass_score` decimal(12,4) NOT NULL COMMENT '试卷及格分',
                               `score_type` varchar(64) NOT NULL COMMENT '赋分类型\\n1.NEED_ASSIGN_SCORE("是","需要且按照公式赋分")\\n2.NO_NEED_ASSIGN_SCORE("否","不需要赋分，标准分即为赋分")\\n3.SPECIAL_ASSIGN_SCORE("特殊赋分","特殊赋分，按照excel表进行赋分")',
                               `create_id` bigint(20) DEFAULT NULL,
                               `create_time` bigint(20) DEFAULT NULL,
                               `update_id` bigint(20) DEFAULT NULL,
                               `update_time` bigint(20) DEFAULT NULL,
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='试卷基础信息表（赋分系数、试卷类型、满分、及格分等）';

-- ----------------------------
-- Table structure for grade_paper_dimension
-- ----------------------------
DROP TABLE IF EXISTS `grade_paper_dimension`;
CREATE TABLE `grade_paper_dimension` (
                                         `id` bigint(20) NOT NULL,
                                         `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                         `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
                                         `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
                                         `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
                                         `dimension_type` varchar(32) NOT NULL COMMENT '维度类型 - 知识，能力，素养',
                                         `code_primary` varchar(32) NOT NULL COMMENT '一级维度编号',
                                         `name_primary` varchar(255) NOT NULL COMMENT '一级维度名称',
                                         `code_second` varchar(32) DEFAULT NULL COMMENT '二级维度编号',
                                         `name_second` varchar(255) DEFAULT NULL COMMENT '二级维度名称',
                                         `interpretation` varchar(255) DEFAULT NULL COMMENT '一级维度术语解释',
                                         `create_id` bigint(20) DEFAULT NULL,
                                         `create_time` bigint(20) DEFAULT NULL,
                                         `update_id` bigint(20) DEFAULT NULL,
                                         `update_time` bigint(20) DEFAULT NULL,
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='分析-考察点维度表';

-- ----------------------------
-- Table structure for grade_paper_struct
-- ----------------------------
DROP TABLE IF EXISTS `grade_paper_struct`;
CREATE TABLE `grade_paper_struct` (
                                      `id` bigint(20) NOT NULL,
                                      `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                      `paper_number` varchar(100) NOT NULL COMMENT '试卷编号',
                                      `paper_type` varchar(16) NOT NULL COMMENT '试卷类型',
                                      `paper_name` varchar(100) NOT NULL COMMENT '试卷名称',
                                      `question_name` varchar(200) NOT NULL COMMENT '大题名称',
                                      `number_type` varchar(32) NOT NULL COMMENT '题号类型（客观题、主观题）',
                                      `big_question_number` varchar(64) NOT NULL COMMENT '大题号',
                                      `small_question_number` varchar(64) NOT NULL COMMENT '小题号(每一课程试卷的唯一标识)',
                                      `big_topic_name` varchar(64) NOT NULL COMMENT '大题名称',
                                      `full_score` decimal(12,4) NOT NULL COMMENT '满分',
                                      `score_rules` varchar(32) DEFAULT NULL COMMENT '计分规则',
                                      `rules_desc` varchar(500) DEFAULT NULL COMMENT '规则说明',
                                      `knowledge_dimension` varchar(255) NOT NULL COMMENT '知识维度',
                                      `ability_dimension` varchar(255) DEFAULT NULL COMMENT '能力维度',
                                      `literacy_dimension` varchar(255) DEFAULT NULL COMMENT '素养维度',
                                      `create_id` bigint(20) DEFAULT NULL,
                                      `create_time` bigint(20) DEFAULT NULL,
                                      `update_id` bigint(20) DEFAULT NULL,
                                      `update_time` bigint(20) DEFAULT NULL,
                                      PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='分析-试卷结构（蓝图）表';

-- ----------------------------
-- Table structure for push_user_error
-- ----------------------------
DROP TABLE IF EXISTS `push_user_error`;
CREATE TABLE `push_user_error` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                   `user_id` bigint(20) DEFAULT NULL COMMENT '逸教云系统用户id',
                                   `account` varchar(100) DEFAULT NULL COMMENT '关联名称（唯一标识）',
                                   `name` varchar(100) DEFAULT NULL COMMENT '名称',
                                   `password` varchar(50) DEFAULT NULL COMMENT '密码',
                                   `role` varchar(100) DEFAULT NULL COMMENT '特殊角色权限\n    MARKER("评卷员权限","Marker","M_"),\n    SUBJECT_HEADER("科组长权限","SubjectHeader","S_"),',
                                   `enable` tinyint(1) DEFAULT NULL COMMENT '是否启用',
                                   `subject_codes` varchar(100) DEFAULT NULL COMMENT '科组长关联科目，科目代码用,分割，例如ASC,WED',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='推送用户异常数据';

-- ----------------------------
-- Table structure for push_user_track
-- ----------------------------
DROP TABLE IF EXISTS `push_user_track`;
CREATE TABLE `push_user_track` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                   `user_id` bigint(20) NOT NULL COMMENT '用户id',
                                   `identify` varchar(50) NOT NULL COMMENT '身份\nSpecialPrivilegeEnum：\nSUBJECT_HEADER("科组长权限","SubjectHeader","S_", 4),\nMARKER("评卷员权限","Marker","M_", 5),',
                                   `operate` varchar(50) NOT NULL COMMENT '操作\nUserPushTrackEnum：\nBIND("绑定")\nUNBIND("解绑")\n',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='用户推送轨迹表';

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_blob_triggers`;
CREATE TABLE `qrtz_blob_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `BLOB_DATA` blob,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                      KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_calendars`;
CREATE TABLE `qrtz_calendars` (
                                  `SCHED_NAME` varchar(120) NOT NULL,
                                  `CALENDAR_NAME` varchar(200) NOT NULL,
                                  `CALENDAR` blob NOT NULL,
                                  PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_cron_triggers`;
CREATE TABLE `qrtz_cron_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `CRON_EXPRESSION` varchar(120) NOT NULL,
                                      `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_fired_triggers`;
CREATE TABLE `qrtz_fired_triggers` (
                                       `SCHED_NAME` varchar(120) NOT NULL,
                                       `ENTRY_ID` varchar(95) NOT NULL,
                                       `TRIGGER_NAME` varchar(200) NOT NULL,
                                       `TRIGGER_GROUP` varchar(200) NOT NULL,
                                       `INSTANCE_NAME` varchar(200) NOT NULL,
                                       `FIRED_TIME` bigint(20) NOT NULL,
                                       `SCHED_TIME` bigint(20) NOT NULL,
                                       `PRIORITY` int(11) NOT NULL,
                                       `STATE` varchar(16) NOT NULL,
                                       `JOB_NAME` varchar(200) DEFAULT NULL,
                                       `JOB_GROUP` varchar(200) DEFAULT NULL,
                                       `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
                                       `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
                                       PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_job_details`;
CREATE TABLE `qrtz_job_details` (
                                    `SCHED_NAME` varchar(120) NOT NULL,
                                    `JOB_NAME` varchar(200) NOT NULL,
                                    `JOB_GROUP` varchar(200) NOT NULL,
                                    `DESCRIPTION` varchar(250) DEFAULT NULL,
                                    `JOB_CLASS_NAME` varchar(250) NOT NULL,
                                    `IS_DURABLE` varchar(1) NOT NULL,
                                    `IS_NONCONCURRENT` varchar(1) NOT NULL,
                                    `IS_UPDATE_DATA` varchar(1) NOT NULL,
                                    `REQUESTS_RECOVERY` varchar(1) NOT NULL,
                                    `JOB_DATA` blob,
                                    PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                    KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                    KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_locks`;
CREATE TABLE `qrtz_locks` (
                              `SCHED_NAME` varchar(120) NOT NULL,
                              `LOCK_NAME` varchar(40) NOT NULL,
                              PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_paused_trigger_grps`;
CREATE TABLE `qrtz_paused_trigger_grps` (
                                            `SCHED_NAME` varchar(120) NOT NULL,
                                            `TRIGGER_GROUP` varchar(200) NOT NULL,
                                            PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_scheduler_state`;
CREATE TABLE `qrtz_scheduler_state` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `INSTANCE_NAME` varchar(200) NOT NULL,
                                        `LAST_CHECKIN_TIME` bigint(20) NOT NULL,
                                        `CHECKIN_INTERVAL` bigint(20) NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simple_triggers`;
CREATE TABLE `qrtz_simple_triggers` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `TRIGGER_NAME` varchar(200) NOT NULL,
                                        `TRIGGER_GROUP` varchar(200) NOT NULL,
                                        `REPEAT_COUNT` bigint(20) NOT NULL,
                                        `REPEAT_INTERVAL` bigint(20) NOT NULL,
                                        `TIMES_TRIGGERED` bigint(20) NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simprop_triggers`;
CREATE TABLE `qrtz_simprop_triggers` (
                                         `SCHED_NAME` varchar(120) NOT NULL,
                                         `TRIGGER_NAME` varchar(200) NOT NULL,
                                         `TRIGGER_GROUP` varchar(200) NOT NULL,
                                         `STR_PROP_1` varchar(512) DEFAULT NULL,
                                         `STR_PROP_2` varchar(512) DEFAULT NULL,
                                         `STR_PROP_3` varchar(512) DEFAULT NULL,
                                         `INT_PROP_1` int(11) DEFAULT NULL,
                                         `INT_PROP_2` int(11) DEFAULT NULL,
                                         `LONG_PROP_1` bigint(20) DEFAULT NULL,
                                         `LONG_PROP_2` bigint(20) DEFAULT NULL,
                                         `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
                                         `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
                                         `BOOL_PROP_1` varchar(1) DEFAULT NULL,
                                         `BOOL_PROP_2` varchar(1) DEFAULT NULL,
                                         PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_triggers`;
CREATE TABLE `qrtz_triggers` (
                                 `SCHED_NAME` varchar(120) NOT NULL,
                                 `TRIGGER_NAME` varchar(200) NOT NULL,
                                 `TRIGGER_GROUP` varchar(200) NOT NULL,
                                 `JOB_NAME` varchar(200) NOT NULL,
                                 `JOB_GROUP` varchar(200) NOT NULL,
                                 `DESCRIPTION` varchar(250) DEFAULT NULL,
                                 `NEXT_FIRE_TIME` bigint(20) DEFAULT NULL,
                                 `PREV_FIRE_TIME` bigint(20) DEFAULT NULL,
                                 `PRIORITY` int(11) DEFAULT NULL,
                                 `TRIGGER_STATE` varchar(16) NOT NULL,
                                 `TRIGGER_TYPE` varchar(8) NOT NULL,
                                 `START_TIME` bigint(20) NOT NULL,
                                 `END_TIME` bigint(20) DEFAULT NULL,
                                 `CALENDAR_NAME` varchar(200) DEFAULT NULL,
                                 `MISFIRE_INSTR` smallint(6) DEFAULT NULL,
                                 `JOB_DATA` blob,
                                 PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
DROP TABLE IF EXISTS `sys_config`;
CREATE TABLE `sys_config` (
                              `id` bigint(20) NOT NULL COMMENT '主键',
                              `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                              `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                              `config_key` varchar(100) NOT NULL COMMENT '参数键名',
                              `config_name` varchar(100) NOT NULL COMMENT '参数名称',
                              `config_value` varchar(100) NOT NULL COMMENT '参数键值',
                              `remark` varchar(100) DEFAULT NULL COMMENT '备注',
                              `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                              `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                              `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='参数表';

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
DROP TABLE IF EXISTS `sys_org`;
CREATE TABLE `sys_org` (
                           `id` bigint(20) NOT NULL COMMENT '主键',
                           `school_id` bigint(20) NOT NULL COMMENT '学校id',
                           `type` varchar(50) NOT NULL COMMENT '类型，SCHOOL：学校，COLLEGE：学院，FACULTY：院系，TEACHING_ROOM：教研室，PRINTING_HOUSE：印刷厂',
                           `code` varchar(20) DEFAULT NULL COMMENT '机构代码',
                           `name` varchar(100) NOT NULL COMMENT '机构名称',
                           `parent_id` bigint(20) DEFAULT NULL COMMENT '上级机构id',
                           `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                           `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                           `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                           `historic_name` varchar(100) DEFAULT NULL COMMENT '历史机构名称',
                           PRIMARY KEY (`id`) USING BTREE,
                           UNIQUE KEY `org_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学校组织架构';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `name` varchar(100) NOT NULL COMMENT '菜单名称',
                                 `url` varchar(100) NOT NULL COMMENT 'URL地址',
                                 `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件，TAB：选项卡',
                                 `parent_id` bigint(20) DEFAULT NULL COMMENT '上级菜单',
                                 `sequence` int(11) NOT NULL COMMENT '序号',
                                 `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
                                 `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
                                 `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 `default_auth` tinyint(4) DEFAULT '1' COMMENT '默认权限，0：否，1：是',
                                 `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
                                 PRIMARY KEY (`id`) USING BTREE,
                                 UNIQUE KEY `privilege_index` (`name`,`url`,`type`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='菜单权限表';

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_role`;
CREATE TABLE `sys_role` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                            `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                            `name` varchar(50) NOT NULL COMMENT '角色名称',
                            `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，SCHOOL_ADMIN：管理员：EXAM_TEACHER：考务老师，QUESTION_TEACHER：命题老师，CUSTOMER：客服人员，PRINTER：印刷人员，CUSTOM：自定义',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                            `default_role` tinyint(4) NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
                            `source` varchar(100) DEFAULT NULL COMMENT '角色来源，ANALYSIS：教研分析',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `role_schoolId_name_idx` (`school_id`,`name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='角色表';

-- ----------------------------
-- Table structure for sys_role_course
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_course`;
CREATE TABLE `sys_role_course` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                   `course_id` bigint(20) NOT NULL COMMENT '课程id',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='角色关联课程';

-- ----------------------------
-- Table structure for sys_role_group_member
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_group_member`;
CREATE TABLE `sys_role_group_member` (
                                         `id` bigint(20) NOT NULL COMMENT '主键',
                                         `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                         `user_id` bigint(20) NOT NULL COMMENT '用户id',
                                         `member_id` bigint(20) NOT NULL COMMENT '成员id',
                                         `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                         PRIMARY KEY (`id`) USING BTREE,
                                         UNIQUE KEY `role_group_roleId_memberId_idx` (`role_id`,`member_id`,`enable`) USING BTREE,
                                         KEY `role_group_member_idx` (`role_id`,`user_id`,`member_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='角色组成员表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_privilege`;
CREATE TABLE `sys_role_privilege` (
                                      `id` bigint(20) NOT NULL COMMENT '主键',
                                      `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                      `privilege_id` bigint(20) NOT NULL COMMENT '菜单id',
                                      `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='角色和菜单关联表';

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
DROP TABLE IF EXISTS `sys_user`;
CREATE TABLE `sys_user` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                            `login_name` varchar(100) NOT NULL COMMENT '用户名',
                            `real_name` varchar(100) NOT NULL COMMENT '姓名',
                            `code` varchar(100) DEFAULT NULL COMMENT '工号',
                            `password` varchar(50) NOT NULL COMMENT '密码',
                            `mobile_number` varchar(11) DEFAULT NULL COMMENT '手机号',
                            `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                            `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `pwd_count` tinyint(4) DEFAULT '0' COMMENT '密码修改次数，默认为0',
                            `pwd_update_time` bigint(20) DEFAULT NULL COMMENT '密码修改时间',
                            `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                            `sync_status` varchar(100) DEFAULT NULL COMMENT '同步状态，NEED_PUSH_AGAIN("需要重新推送"),',
                            `historic_name` varchar(100) DEFAULT NULL COMMENT '历史用户名称',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `user_schoolId_loginName_idx` (`school_id`,`login_name`) USING BTREE,
                            UNIQUE KEY `uq_school_id_mobile_number` (`school_id`,`mobile_number`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='用户表';

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_user_role`;
CREATE TABLE `sys_user_role` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `user_id` bigint(20) NOT NULL COMMENT '用户id',
                                 `role_id` bigint(20) NOT NULL COMMENT '角色id',
                                 `privilege_id` bigint(20) DEFAULT NULL COMMENT '权限id',
                                 `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='用户和角色关联表';

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
DROP TABLE IF EXISTS `t_b_session`;
CREATE TABLE `t_b_session` (
                               `id` varchar(100) NOT NULL COMMENT '主键',
                               `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                               `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                               `identity` varchar(100) NOT NULL COMMENT '用户标识',
                               `type` varchar(50) NOT NULL COMMENT '用户类型',
                               `source` varchar(50) NOT NULL COMMENT '访问来源',
                               `platform` varchar(50) NOT NULL COMMENT '设备分类',
                               `device_id` varchar(100) NOT NULL COMMENT '设备标识',
                               `address` varchar(100) NOT NULL COMMENT '登录IP地址',
                               `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
                               `last_access_time` bigint(20) DEFAULT NULL COMMENT '最近访问时间',
                               `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '令牌更新时间',
                               `expire_time` bigint(20) DEFAULT NULL COMMENT '令牌强制失效时间',
                               `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_sync_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sync_task`;
CREATE TABLE `t_b_sync_task` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                 `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                 `type` varchar(50) NOT NULL COMMENT '推送类型，USER_PUSH：用户推送',
                                 `third_relate_id` bigint(20) DEFAULT NULL COMMENT '第三方系统关联ID（目前只有云阅卷）',
                                 `object_id` bigint(20) DEFAULT NULL COMMENT '印刷计划id',
                                 `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
                                 `summary` text COMMENT '实时摘要信息',
                                 `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint(20) DEFAULT NULL COMMENT '修改人',
                                 `update_time` bigint(20) DEFAULT NULL COMMENT '修改时间',
                                 `remark` mediumtext COMMENT '备注',
                                 `reset_count` int(11) DEFAULT '0' COMMENT '重试次数',
                                 `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
                                 `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='云阅卷同步任务表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_task`;
CREATE TABLE `t_b_task` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `type` varchar(50) NOT NULL COMMENT '任务类型，USER_IMPORT：用户导入，\nQUESTION_MISSION_BATCH_CREATE：批量新建命题任务，\nSAMPLE_EXPORT：导出审核样本，\nPAPER_DOWNLOAD：卷库下载，\nEXAMINATION_IMPORT：考务数据导入，\nEXAMINATION_EXPORT：考务数据导出，\nPRINT_PDF_DOWNLOAD：批量下载pdf，\nCREATE_PDF：生成pdf，\nCOURSE_IMPORT：课程导入',
                            `entity_id` bigint(20) DEFAULT NULL COMMENT '关联业务对象id',
                            `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
                            `summary` text COMMENT '实时摘要信息',
                            `progress` double DEFAULT NULL COMMENT '执行进度',
                            `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                            `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                            `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
                            `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
                            `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
                            `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                            `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                            `print_plan_id` bigint(20) DEFAULT NULL COMMENT '印刷计划id',
                            `remark` mediumtext COMMENT '备注',
                            `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
                            `reset_count` int(11) DEFAULT '0' COMMENT '重试次数',
                            `version` int(11) DEFAULT '0' COMMENT '更新版本号',
                            `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_c_statistics
-- ----------------------------
DROP TABLE IF EXISTS `t_c_statistics`;
CREATE TABLE `t_c_statistics` (
                                  `id` bigint(20) NOT NULL COMMENT '主键',
                                  `exam_id` bigint(20) NOT NULL COMMENT '考试id',
                                  `exam_task_detail_ids` mediumtext COMMENT '命题任务明细id',
                                  `exam_detail_course_ids` mediumtext COMMENT '考务数据课程id',
                                  `college_id` bigint(20) NOT NULL COMMENT '学院id',
                                  `college_name` varchar(100) NOT NULL COMMENT '学院名称',
                                  `course_name` varchar(100) NOT NULL COMMENT '课程名称',
                                  `course_code` varchar(100) NOT NULL COMMENT '课程代码',
                                  `teaching_room_id` bigint(20) NOT NULL COMMENT '教研室id',
                                  `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
                                  `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
                                  `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
                                  `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
                                  `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
                                  `print_plan_id` bigint(20) DEFAULT NULL COMMENT '印刷计划id',
                                  `batch_no` varchar(100) NOT NULL COMMENT '批次号',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  `print_sum` varchar(100) DEFAULT NULL COMMENT '总印份数',
                                  `status` varchar(50) DEFAULT NULL COMMENT '完成状态，FINISH：已完成，UN_FINISH：未完成',
                                  `exam_detail_id` bigint(20) DEFAULT NULL COMMENT '命题明细id',
                                  `clazz_content` mediumtext COMMENT '班级详细信息内容',
                                  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_c_statistics_temp
-- ----------------------------
DROP TABLE IF EXISTS `t_c_statistics_temp`;
CREATE TABLE `t_c_statistics_temp` (
                                       `id` bigint(20) NOT NULL COMMENT '主键',
                                       `exam_id` bigint(20) NOT NULL COMMENT '考试id',
                                       `exam_task_detail_id` bigint(20) DEFAULT NULL COMMENT '命题任务明细id',
                                       `exam_detail_course_id` bigint(20) DEFAULT NULL COMMENT '考务数据课程id',
                                       `college_id` bigint(20) NOT NULL COMMENT '学院id',
                                       `college_name` varchar(100) NOT NULL COMMENT '学院名称',
                                       `course_name` varchar(100) NOT NULL COMMENT '课程名称',
                                       `course_code` varchar(100) NOT NULL COMMENT '课程代码',
                                       `teaching_room_id` bigint(20) NOT NULL COMMENT '教研室id',
                                       `teaching_room_name` varchar(100) NOT NULL COMMENT '教研室名称',
                                       `teacher_name` mediumtext NOT NULL COMMENT '任课老师名称',
                                       `clazz_ids` varchar(100) NOT NULL COMMENT '班级ids',
                                       `clazz_name` varchar(100) NOT NULL COMMENT '班级名称',
                                       `paper_number` varchar(100) DEFAULT NULL COMMENT '试卷编号',
                                       `print_plan_id` bigint(20) DEFAULT NULL COMMENT '印刷计划id',
                                       `batch_no` varchar(100) NOT NULL COMMENT '批次号',
                                       `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                       `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                       `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                       `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                       `data_source` varchar(50) DEFAULT NULL COMMENT '数据来源，IMPORT：导入，JOIN：关联查询的数据',
                                       `exam_detail_id` bigint(20) DEFAULT NULL COMMENT '命题明细id',
                                       PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='命题统计表';

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow`;
CREATE TABLE `t_f_custom_flow` (
                                   `id` bigint(20) NOT NULL COMMENT '主键',
                                   `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                   `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                   `name` varchar(100) NOT NULL COMMENT '流程名称',
                                   `model_type` varchar(50) NOT NULL COMMENT '模式类型，USER_FIXED：指定人员,APPROVE_SET：发起人指定',
                                   `type` varchar(100) NOT NULL COMMENT '流程类型，ELECTRON_FLOW：电子交卷审核，QUESTION_ELECTRON_FLOW：题库试题提交审核，PAPER_ELECTRON_FLOW：题库试卷审核',
                                   `enable` tinyint(4) DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                   `publish` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
                                   `object_data` mediumtext NOT NULL COMMENT '自定义流程源数据',
                                   `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                   `version` bigint(20) NOT NULL DEFAULT '1' COMMENT '版本号',
                                   `flow_key` varchar(100) DEFAULT NULL COMMENT '流程key',
                                   `act_flow_id` varchar(100) DEFAULT NULL COMMENT '流程版本id',
                                   `flow_process_var` mediumtext COMMENT '流程过程变量数据',
                                   PRIMARY KEY (`id`) USING BTREE,
                                   UNIQUE KEY `custom_flow_schoolId_type_idx` (`school_id`,`type`,`enable`,`publish`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='自定义流程表';

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_entity`;
CREATE TABLE `t_f_custom_flow_entity` (
                                          `id` bigint(20) NOT NULL COMMENT '主键',
                                          `t_f_custom_flow_id` bigint(20) NOT NULL COMMENT '自定义流程id',
                                          `flow_id` bigint(20) DEFAULT NULL COMMENT '流程id',
                                          `flow_process_var` mediumtext NOT NULL COMMENT '流程过程变量',
                                          `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                          `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                          `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                          `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                          PRIMARY KEY (`id`) USING BTREE,
                                          KEY `flow_entity_flowId_idx` (`flow_id`),
                                          KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='自定义流程实体数据';

-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_approve`;
CREATE TABLE `t_f_flow_approve` (
                                    `id` bigint(20) NOT NULL COMMENT '主键',
                                    `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                    `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                    `flow_id` bigint(20) NOT NULL COMMENT '流程id',
                                    `approve_id` bigint(20) DEFAULT NULL COMMENT '审核id',
                                    `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
                                    `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                    `setup` tinyint(4) NOT NULL COMMENT '计划步骤',
                                    `approve_setup` int(11) DEFAULT NULL COMMENT '审批步骤',
                                    `approve_max_setup` int(11) DEFAULT NULL COMMENT '最大审批步骤',
                                    `transfer_assignee` mediumtext COMMENT '流程转办',
                                    `all_assignee` mediumtext COMMENT '会签审批人',
                                    `reject_link` mediumtext COMMENT '驳回路线',
                                    `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                    PRIMARY KEY (`id`) USING BTREE,
                                    KEY `flow_approve_idx` (`school_id`,`flow_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='流程审核表';

-- ----------------------------
-- Table structure for t_f_flow_join
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_join`;
CREATE TABLE `t_f_flow_join` (
                                 `id` bigint(20) NOT NULL COMMENT '主键',
                                 `flow_entity_id` bigint(20) NOT NULL COMMENT '自定义流程实体id',
                                 `object_table` varchar(100) NOT NULL COMMENT '实体表',
                                 `object_id` bigint(20) NOT NULL COMMENT '实体id',
                                 `flow_id` bigint(20) NOT NULL COMMENT '流程id',
                                 `level` int(11) NOT NULL COMMENT '流程等级',
                                 `type` varchar(50) NOT NULL COMMENT '类型，NEW：新增，RESTART：重新提交',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 PRIMARY KEY (`id`) USING BTREE,
                                 KEY `task_flowId_idx` (`flow_id`) USING BTREE,
                                 KEY `task_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='实体和流程关联表';

-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_log`;
CREATE TABLE `t_f_flow_log` (
                                `id` bigint(20) NOT NULL COMMENT '主键',
                                `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                `flow_id` bigint(20) NOT NULL COMMENT '流程id',
                                `object_table` varchar(100) DEFAULT NULL COMMENT '实体表',
                                `object_id` bigint(20) NOT NULL COMMENT '实体id',
                                `approve_id` bigint(20) DEFAULT NULL COMMENT '审核人id',
                                `approve_setup` tinyint(4) DEFAULT NULL COMMENT '审核步骤',
                                `approve_remark` mediumtext COMMENT '审核意见',
                                `enable` tinyint(4) NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                `approve_operation` varchar(50) NOT NULL COMMENT '审批操作，SUBMIT：提交，APPROVE：审批，REJECT：驳回，END：终止',
                                `pend_approve_id` mediumtext COMMENT '待审核人id',
                                `paper_attachment_id` mediumtext COMMENT '试卷附件',
                                `approve_multi_setup` int(11) DEFAULT NULL COMMENT '审批步骤',
                                `approve_user_approve_type` varchar(30) DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
                                PRIMARY KEY (`id`) USING BTREE,
                                KEY `flow_log_idx` (`school_id`,`flow_id`) USING BTREE,
                                KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='流程流水表';

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
DROP TABLE IF EXISTS `t_g_error`;
CREATE TABLE `t_g_error` (
                             `id` bigint(20) NOT NULL COMMENT '主键',
                             `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                             `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                             `obj_id` bigint(20) DEFAULT NULL COMMENT '实体id',
                             `summary` mediumtext COMMENT '实时摘要信息',
                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='全局异常错误信息表';

-- ----------------------------
-- Table structure for t_m_mq_message
-- ----------------------------
DROP TABLE IF EXISTS `t_m_mq_message`;
CREATE TABLE `t_m_mq_message` (
                                  `id` varchar(50) NOT NULL COMMENT '主键',
                                  `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                  `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                  `topic` varchar(100) NOT NULL COMMENT '消息topic',
                                  `tag` varchar(100) NOT NULL COMMENT '消息tag',
                                  `body` mediumtext COMMENT '消息内容',
                                  `type` varchar(50) NOT NULL COMMENT '消息类型',
                                  `obj_id` varchar(50) NOT NULL COMMENT '关联业务id',
                                  `obj_name` varchar(100) DEFAULT NULL COMMENT '关联业务名称',
                                  `ack` tinyint(4) NOT NULL COMMENT '消息ack',
                                  `sequence` int(11) DEFAULT NULL COMMENT '消息序号',
                                  `properties` varchar(1000) DEFAULT NULL COMMENT '扩展类型',
                                  `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                                  `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                  `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                  `timestamp` bigint(20) NOT NULL COMMENT '时间戳',
                                  `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='mq消息';

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
DROP TABLE IF EXISTS `t_s_auth`;
CREATE TABLE `t_s_auth` (
                            `id` bigint(20) NOT NULL COMMENT '主键',
                            `school_id` bigint(20) NOT NULL COMMENT '学校id',
                            `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
                            `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
                            `description` text COMMENT '离线授权证书',
                            `type` varchar(16) NOT NULL COMMENT '授权类型',
                            `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                            `expire_time` bigint(20) DEFAULT NULL COMMENT '过期时间',
                            `file` mediumblob COMMENT '文件数据',
                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='激活授权配置表';

-- ----------------------------
-- Table structure for t_school_privilege
-- ----------------------------
DROP TABLE IF EXISTS `t_school_privilege`;
CREATE TABLE `t_school_privilege` (
                                      `id` bigint(20) NOT NULL COMMENT '主键',
                                      `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                      `privilege_id` bigint(20) NOT NULL COMMENT '菜单权限id',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='学校菜单权限表';

-- ----------------------------
-- Table structure for t_sync_exam_student_score
-- ----------------------------
DROP TABLE IF EXISTS `t_sync_exam_student_score`;
CREATE TABLE `t_sync_exam_student_score` (
                                             `id` bigint(20) NOT NULL COMMENT '主键',
                                             `school_id` bigint(20) DEFAULT NULL COMMENT '学校id',
                                             `org_id` bigint(20) DEFAULT NULL COMMENT '机构id',
                                             `semester_id` bigint(20) DEFAULT NULL COMMENT '学期id',
                                             `major_id` bigint(20) DEFAULT NULL COMMENT '专业id',
                                             `clazz_id` bigint(20) DEFAULT NULL COMMENT '班级id',
                                             `exam_id` bigint(20) DEFAULT NULL COMMENT '云阅卷考试id',
                                             `exam_code` varchar(100) DEFAULT NULL COMMENT '云阅卷考试标识',
                                             `exam_number` varchar(100) DEFAULT NULL COMMENT '准考证号',
                                             `student_code` varchar(100) DEFAULT NULL COMMENT '学号',
                                             `name` varchar(100) NOT NULL COMMENT '姓名',
                                             `subject_code` varchar(100) DEFAULT NULL COMMENT '科目代码',
                                             `subject_name` varchar(100) DEFAULT NULL COMMENT '科目名称',
                                             `paper_type` varchar(50) DEFAULT NULL COMMENT '试卷类型',
                                             `college` varchar(100) DEFAULT NULL COMMENT '学院',
                                             `clazz_name` varchar(100) DEFAULT NULL COMMENT '班级',
                                             `teacher` varchar(100) DEFAULT NULL COMMENT '教师',
                                             `status` tinyint(4) DEFAULT NULL COMMENT '考生状态，1-正常，2-缺考(包含未上传)，3-违纪',
                                             `sheet_urls` mediumtext COMMENT '原卷url',
                                             `local_sheet_urls` mediumtext COMMENT '从云阅卷下载下来的原图url',
                                             `total_score` double DEFAULT NULL COMMENT '总分',
                                             `objective_score` double DEFAULT NULL COMMENT '客观题分数',
                                             `subjective_score` double DEFAULT NULL COMMENT '主观题分数',
                                             `sync_data` mediumtext COMMENT '同步数据json',
                                             `trajectory_urls` varchar(500) DEFAULT NULL COMMENT '轨迹图地址',
                                             `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                             `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                             PRIMARY KEY (`id`) USING BTREE,
                                             KEY `score_schoolId_idx` (`school_id`,`semester_id`,`org_id`,`major_id`,`clazz_id`,`student_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='同步考生成绩表';

-- ----------------------------
-- Table structure for teach_clazz
-- ----------------------------
DROP TABLE IF EXISTS `teach_clazz`;
CREATE TABLE `teach_clazz` (
                               `id` bigint(20) NOT NULL COMMENT '教学班表主键',
                               `school_id` bigint(20) NOT NULL COMMENT '学校id',
                               `teach_clazz_code` varchar(20) DEFAULT NULL COMMENT '教学班编号',
                               `teach_clazz_name` varchar(50) NOT NULL COMMENT '教学班名称',
                               `user_id` bigint(20) NOT NULL COMMENT '用户id',
                               `teach_course_id` bigint(20) NOT NULL COMMENT '教学课程主键',
                               `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                               `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                               `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `teach_clazz_unique` (`school_id`,`teach_course_id`,`teach_clazz_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='教学班表';

-- ----------------------------
-- Table structure for teach_course
-- ----------------------------
DROP TABLE IF EXISTS `teach_course`;
CREATE TABLE `teach_course` (
                                `id` bigint(20) NOT NULL,
                                `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                `basic_course_id` bigint(20) NOT NULL COMMENT '基础课程id',
                                `user_id` bigint(20) NOT NULL COMMENT '创建此教学课程的用户id',
                                `enable` tinyint(1) DEFAULT NULL COMMENT '是否删除',
                                `create_id` bigint(20) DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint(20) DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `teach_course_unique` (`school_id`,`basic_course_id`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='教学课程表';

-- ----------------------------
-- Table structure for teach_student
-- ----------------------------
DROP TABLE IF EXISTS `teach_student`;
CREATE TABLE `teach_student` (
                                 `id` bigint(20) NOT NULL COMMENT '教学学生关系表主键',
                                 `school_id` bigint(20) NOT NULL COMMENT '学校id',
                                 `student_id` bigint(20) NOT NULL COMMENT '学生id',
                                 `teach_clazz_id` bigint(20) NOT NULL COMMENT '教学班级id',
                                 `teach_course_id` bigint(20) NOT NULL COMMENT '教学课程id（冗余）',
                                 `user_id` bigint(20) NOT NULL COMMENT '用户id（冗余）',
                                 `create_id` bigint(20) DEFAULT NULL COMMENT '创建人',
                                 `create_time` bigint(20) DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint(20) DEFAULT NULL COMMENT '更新人',
                                 `update_time` bigint(20) DEFAULT NULL COMMENT '更新时间',
                                 PRIMARY KEY (`id`),
                                 UNIQUE KEY `teach_student_unique` (`school_id`,`student_id`,`teach_clazz_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='教学学生关系表';

INSERT INTO `sys_privilege` VALUES (1, '基础配置', 'base', 'MENU', NULL, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (2, '用户管理', 'user', 'MENU', 1, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (3, '配置管理', 'rule', 'MENU', 1, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (4, '组织架构', 'OrganizationManage', 'MENU', 2, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (5, '用户管理', 'UserManage', 'MENU', 2, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (6, '角色管理', 'RoleManage', 'MENU', 2, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (7, '权限管理', 'MenuManage', 'MENU', 129, 6, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (8, '通用规则', 'CommonRule', 'MENU', 3, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (9, '题卡规则管理', 'CardRuleManage', 'MENU', 3, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (10, '通卡模板', 'CommonCardTemplate', 'MENU', 3, 3, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (11, '变量印品模板', 'ParamPrintTemplate', 'MENU', 3, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (12, '普通印品模板', 'CommonPrintTemplate', 'MENU', 3, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (13, '课程管理', 'CourseManage', 'MENU', 356, 7, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (14, '考试中心', 'exam', 'MENU', NULL, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (15, '我的工作台', 'work', 'MENU', 14, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (16, '待办任务', 'WaitTask', 'MENU', 15, 1, NULL, '477,565,566,417,420,421,441,442,640,641', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (38, '印刷管理', 'print', 'MENU', 14, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (39, '命题任务管理', 'ExamTaskManage', 'MENU', 652, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (40, '入库申请', 'TaskApplyManage', 'MENU', 652, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (41, '入库审核', 'TaskReviewManage', 'MENU', 652, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (42, '卷库查询', 'TaskPaperManage', 'MENU', 652, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (43, '印刷计划管理', 'PrintPlanManage', 'MENU', 38, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (44, '考务数据导入', 'BusinessDataExport', 'MENU', 38, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (45, '考务明细查询', 'BusinessDataDetail', 'MENU', 38, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (46, '关联试卷管理', 'PlanLinkPaper', 'MENU', 38, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (47, '印刷任务管理', 'PrintTaskManage', 'MENU', 38, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (48, '印刷计划管理-查询', '/api/admin/exam/print/list', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (49, '印刷计划管理-按类型查询模板', '/api/admin/exam/print/get_template', 'URL', 43, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (50, '印刷计划管理-新增/修改', '/api/admin/exam/print/save', 'URL', 43, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (51, '印刷计划管理-删除', '/api/admin/exam/print/remove', 'URL', 43, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (52, '考务数据导入-查询', '/api/admin/exam/print/data_list', 'URL', 44, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (53, '考务数据导入-汇总数据查询', '/api/admin/exam/print/data_total', 'URL', 44, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (54, '考务数据导入-导入', '/api/admin/exam/print/data_import', 'URL', 44, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (55, '考务数据导入-查看明细', '/api/admin/exam/print/data_detail_list', 'URL', 44, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (56, '关联试卷管理-查询', '/api/admin/exam/print/relate_list', 'URL', 46, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (57, '关联试卷管理-关联/更换试卷', '/api/admin/exam/print/relate_update', 'URL', 46, 2, 'AUTH', '58', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (58, '关联试卷管理-查询所有试卷编号/关联卷型', '/api/admin/exam/print/relate_get_paper_numbers', 'URL', 46, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (59, '印刷任务管理-查询', '/api/admin/exam/print/task_list', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (60, '印刷任务管理-提交印刷', '/api/admin/exam/print/task_submit', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (61, '印刷任务管理-撤回提交', '/api/admin/exam/print/task_cancel', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (62, '印刷任务管理-汇总数据查询', '/api/admin/exam/print/task_total_data', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (63, '命题任务管理-查询', '/api/admin/exam/task/list', 'URL', 39, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (64, '命题任务管理-更改/指派命题老师', '/api/admin/exam/task/assign_user', 'URL', 39, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (65, '命题任务管理-启用/禁用', '/api/admin/exam/task/enable', 'URL', 39, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (66, '命题任务管理-新建', '/api/admin/exam/task/save', 'URL', 39, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (67, '命题任务管理-批量新建-文件导入', '/api/admin/exam/task/import', 'URL', 39, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (68, '命题任务管理-批量新建', '/api/admin/exam/task/save_batch', 'URL', 39, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (69, '命题任务管理-审核历史', '/api/admin/exam/task/review_list', 'URL', 39, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (70, '入库申请-查询', '/api/admin/exam/task/apply_list', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (71, '入库申请-提交/暂存', '/api/admin/exam/task/apply_save', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (72, '入库申请-撤回/重新申请', '/api/admin/exam/task/apply_status', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (73, '入库审核-查询未审核', '/api/admin/exam/task/review_list_unaudited', 'URL', 41, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (74, '入库审核-审核', '/api/admin/exam/task/review_save', 'URL', 41, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (75, '入库审核-批量审核', '/api/admin/exam/task/review_save_batch', 'URL', 41, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (76, '卷库查询-查询', '/api/admin/exam/task/paper_list', 'URL', 42, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (77, '用户管理-查询', '/api/admin/sys/user/list', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (78, '用户管理-新增/修改', '/api/admin/sys/user/save', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (79, '用户管理-启用/禁用', '/api/admin/sys/user/enable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (80, '用户管理-重置密码', '/api/admin/sys/user/reset_password', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (81, '用户管理-导入', '/api/admin/sys/user/import', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (82, '用户管理-绑定角色', '/api/admin/sys/user/bind_roles', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (83, '角色管理-查询', '/api/admin/sys/role/list', 'URL', 6, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (84, '角色管理-删除', '/api/admin/sys/role/remove', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (85, '角色管理-新增/修改', '/api/admin/sys/role/save', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (86, '角色管理-启用/禁用', '/api/admin/sys/role/enable', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (87, '用户管理-用户已绑定角色列表', '/api/admin/sys/role/get_user_roles', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (88, '权限管理-查询', '/api/admin/sys/privilege/list', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (89, '权限管理-新增/修改', '/api/admin/sys/privilege/save', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (90, '权限管理-删除', '/api/admin/sys/privilege/remove', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (91, '角色管理-角色已绑定权限列表', '/api/admin/sys/privilege/get_role_privileges', 'URL', 6, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (92, '组织机构-查询', '/api/admin/sys/org/list', 'URL', 4, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (93, '组织机构-新增/修改', '/api/admin/sys/org/save', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (94, '组织机构-删除', '/api/admin/sys/org/remove', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (95, '组织机构-启用/禁用', '/api/admin/sys/org/enable', 'URL', 4, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (96, '课程管理-查询', '/api/admin/basic/course/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (97, '课程管理-新增/修改', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (98, '课程管理-删除', '/api/admin/basic/course/remove', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (99, '考务规则-查询', '/api/admin/basic/exam_rule/list', 'URL', 8, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (100, '考务规则-新增/修改', '/api/admin/basic/exam_rule/save', 'URL', 8, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (101, '题卡规则-查询', '/api/admin/basic/card_rule/list', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (102, '题卡规则-查询单个', '/api/admin/basic/card_rule/get_one', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (103, '题卡规则-新增/修改', '/api/admin/basic/card_rule/save', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (104, '题卡规则-启用/禁用', '/api/admin/basic/card_rule/enable', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (105, '通用模板-查询', '/api/admin/basic/template/list', 'URL', 10, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (106, '通用模板-新增/修改', '/api/admin/basic/template/save', 'URL', 10, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (107, '通用模板-启用/禁用', '/api/admin/basic/template/enable', 'URL', 10, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (112, '系统设置', 'data', 'MENU', 1, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (113, '任务管理', 'DataTaskManage', 'MENU', 112, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (114, '任务管理-查询', '/api/admin/data/task/query', 'URL', 113, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (117, '命题待办接口', '/api/admin/work/exam/task/ready', 'URL', 16, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (118, '命题待办计数接口', '/api/admin/work/exam/task/ready_count', 'URL', 16, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (119, '命题分配待办接口', '/api/admin/work/exam/task/new', 'URL', 16, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (120, '命题分配待办计数接口', '/api/admin/work/exam/task/new_count', 'URL', 16, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (121, '审核待办接口', '/api/admin/work/exam/task/submit', 'URL', 16, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (122, '审核待办计数接口', '/api/admin/work/exam/task/submit_count', 'URL', 16, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (123, '变量印品模板-查询', '/api/admin/basic/template/list', 'URL', 11, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (124, '变量印品模板-新增/修改', '/api/admin/basic/template/save', 'URL', 11, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (125, '变量印品模板-启用/禁用', '/api/admin/basic/template/enable', 'URL', 11, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (126, '普通印品模板-查询', '/api/admin/basic/template/list', 'URL', 12, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (127, '普通印品模板-新增/修改', '/api/admin/basic/template/save', 'URL', 12, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (128, '普通印品模板-启用/禁用', '/api/admin/basic/template/enable', 'URL', 12, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (129, '公共接口', 'common', 'MENU', NULL, 31, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (130, '用户登录相关', 'login', 'MENU', 129, 1, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (131, '用户登录', '/api/admin/common/login', 'URL', 130, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (132, '查询用户权限', '/api/admin/common/get_menu', 'URL', 130, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (133, '发送验证码', '/api/admin/common/get_verify_code', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (134, '修改密码', '/api/admin/sys/user/update_password', 'URL', 130, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (135, '用户登出', '/api/admin/common/logout', 'URL', 130, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (136, '模糊查询', 'query', 'MENU', 129, 2, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (137, '模糊查询-印刷计划', '/api/admin/exam/print/query', 'URL', 136, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (138, '模糊查询-题卡规则', '/api/admin/basic/card_rule/query', 'URL', 136, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (139, '模糊查询-课程', '/api/admin/basic/course/query', 'URL', 136, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (140, '模糊查询-试卷编号', '/api/admin/exam/task/paper_number_query', 'URL', 136, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (141, '模糊查询-命题老师', '/api/admin/exam/task/user_query', 'URL', 136, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (142, '模糊查询-考点', '/api/admin/exam/print/data_place_query', 'URL', 136, 6, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (143, '模糊查询-考场', '/api/admin/exam/print/data_room_query', 'URL', 136, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (144, '文件相关', 'file', 'MENU', 129, 3, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (145, '文件上传', '/api/admin/common/file/upload', 'URL', 144, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (146, '文件查询', '/api/admin/common/file/get_one', 'URL', 144, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (147, '文件下载', '/api/admin/common/file/download', 'URL', 144, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (148, '文件预览', '/api/admin/common/file/preview', 'URL', 144, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (149, '系统相关', 'sys', 'MENU', 129, 4, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (150, '查询枚举类型', '/api/admin/common/get_enums', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (151, '查询学校列表', '/api/admin/common/school/list', 'URL', 149, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (152, '查询系统参数', '/api/admin/common/sys_config/get_one', 'URL', 149, 3, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (153, '根据学校代码查询学校信息', '/api/admin/common/school/query_by_school_code', 'URL', 149, 4, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (154, '其他', 'other', 'MENU', 129, 5, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (155, '用户导入（异步）', '/api/admin/sys/user/async/import', 'URL', 154, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (156, '用户导出', '/api/admin/sys/user/export', 'URL', 154, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (157, '考务数据导入-模板下载', '/api/admin/exam/print/template_download', 'URL', 44, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (158, '考务数据导入-结果导出', '/api/admin/exam/print/data_export', 'URL', 44, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (159, '考务明细查询-查询', '/api/admin/exam/print/data_detail', 'URL', 45, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (160, '印刷任务管理-重新提交印刷', '/api/admin/exam/print/task_resubmit', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (161, '印刷任务管理-查看PDF', '/api/admin/exam/print/task_view_pdf', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (162, '印刷任务管理-批量下载PDF', '/api/admin/exam/print/task_download_pdf', 'URL', 47, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (163, '入库审核-导出审核样本', '/api/admin/exam/task/review_export', 'URL', 41, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (164, '卷库查询-下载试卷', '/api/admin/exam/task/paper_download', 'URL', 42, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (165, '卷库查询-试卷启用/禁用', '/api/admin/exam/task/paper_enable', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (166, '入库审核-查询已审核', '/api/admin/exam/task/review_list_audited', 'URL', 41, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (167, '入库申请-查询可选择题卡', '/api/admin/exam/card/select_card_list', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (168, '入库申请-查询申请详情', '/api/admin/exam/task/apply_get_one', 'URL', 40, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (169, '命题任务管理-根据课程查询命题老师', '/api/admin/sys/user/user_list', 'URL', 39, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (170, '客服制卡', 'customer', 'MENU', NULL, 15, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (171, '客服制卡', 'customerManage', 'MENU', 170, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (172, '题卡审核', 'CustomerCard', 'MENU', 171, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (173, '客服制卡-查询', '/api/admin/exam/card/cust_list', 'URL', 172, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (174, '入库申请-申请客服制卡', '/api/admin/exam/card/cust_save', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (175, '题卡管理', 'card', 'MENU', NULL, 16, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (176, '题卡管理-查询题卡规则', '/api/admin/basic/card_rule/get_one', 'URL', 175, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (177, '题卡管理-查询题卡详情', '/api/admin/exam/card/get_one', 'URL', 175, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (178, '题卡管理-保存/提交', '/api/admin/exam/card/save', 'URL', 175, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (179, '查询考务规则', '/api/admin/basic/exam_rule/list', 'URL', 149, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (180, '角色管理-查询可设置权限列表', '/api/admin/sys/privilege/list_auth', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (181, '超管中心', 'admin', 'MENU', 129, 7, NULL, NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (182, '查询客服', '/api/admin/sys/user/list_customer', 'URL', 181, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (183, '新增客服', '/api/admin/sys/user/save_customer', 'URL', 181, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (184, '用户管理-查询用户可绑定角色', '/api/admin/sys/role/list_to_user', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (185, '客服制卡-批量下载文件', '/api/admin/exam/card/download_files', 'URL', 172, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (186, '印刷任务管理-查看印品', '/api/admin/exam/print/template_view', 'URL', 47, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (187, '客户端登录', '/api/admin/client/user/login', 'URL', NULL, 1, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (188, '考务明细查询-学生明细', '/api/admin/exam/print/get_student_detail', 'URL', 45, 2, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (189, '卷库查询-修改', '/api/admin/exam/task/paper_update', 'URL', 42, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (190, '任务管理-重新生成pdf', '/api/admin/data/task/reset_create_pdf', 'URL', 113, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (191, '卷库查询-批量下载试卷PDF、题卡', '/api/admin/exam/task/paper_card_download_pdf', 'URL', 42, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (192, '读取模板内容', '/api/admin/basic/template/read_content', 'URL', 10, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (195, '获取服务器时间', '/api/admin/common/get_system_time', 'URL', 129, 2, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (198, '数据同步', '/api/admin/exam/print/sync_data_cloud', 'URL', 47, 9, 'NO_AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (199, '客户端', 'client', 'MENU', NULL, 22, NULL, '200,201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216', 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (200, '试卷打样-查询列表', '/api/admin/client/paper_try/list', 'URL', 199, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (201, '试卷打样-查看/试印/重印', '/api/admin/client/paper_try/print', 'URL', 199, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (202, '试卷打样-批量试印', '/api/admin/client/paper_try/print_batch', 'URL', 199, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (203, '试卷打样-标记合格状态', '/api/admin/client/paper_try/tag_pass', 'URL', 199, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (204, '印刷管理-查询列表', '/api/admin/client/print/task_list', 'URL', 199, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (205, '印刷管理-导出', '/api/admin/client/print/task_list_export', 'URL', 199, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (206, '印刷管理-汇总数据查询', '/api/admin/client/print/task_total_data', 'URL', 199, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (207, '印刷管理-查看', '/api/admin/client/print/preview', 'URL', 199, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (208, '印刷管理-印刷/缓存数据', '/api/admin/client/print/get_print_data', 'URL', 199, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (209, '印刷管理-批量缓存', '/api/admin/client/print/get_print_data_batch', 'URL', 199, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (210, '印刷管理-缓存后更新状态', '/api/admin/client/print/update_download', 'URL', 199, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (211, '印刷管理-校验', '/api/admin/client/print/validate_data', 'URL', 199, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (212, '印刷管理-更新打印进度', '/api/admin/client/print/update_progress', 'URL', 199, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (213, '重打-查询考生列表', '/api/admin/client/print/list_student', 'URL', 199, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (214, '重打-内容查询', '/api/admin/client/print/get_reprint_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (215, '统计查询-查询列表', '/api/admin/client/print/statistics_list', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (216, '统计查询-汇总数据查询', '/api/admin/client/print/statistics_total_data', 'URL', 199, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (217, '用户管理-查询', 'Select', 'BUTTON', 5, 1, 'AUTH', '77', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (218, '用户管理-添加用户', 'Add', 'BUTTON', 5, 2, 'AUTH', '78', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (219, '用户管理-编辑', 'Edit', 'LINK', 5, 1, 'AUTH', '78', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (220, '用户管理-启用&禁用', 'Enable', 'LINK', 5, 2, 'AUTH', '79', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (221, '用户管理-列表', 'List', 'LIST', 5, 1, 'AUTH', '77', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (222, '用户管理-查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', '184,87', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (223, '用户管理-重置密码', 'Reset', 'LINK', 5, 3, 'AUTH', '80', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (224, '组织架构-添加', 'Add', 'BUTTON', 4, 1, 'AUTH', '93', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (225, '组织架构-+子部门', 'Add', 'LINK', 4, 1, 'AUTH', '93', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (226, '组织架构-编辑', 'Edit', 'LINK', 4, 2, 'AUTH', '93', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (227, '组织架构-删除', 'Delete', 'LINK', 4, 3, 'AUTH', '94', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (228, '组织架构-树形列表', 'List', 'LIST', 4, 1, 'AUTH', '92', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (229, '角色管理-查询', 'Select', 'BUTTON', 6, 1, 'AUTH', '83,88', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (230, '角色管理-添加角色', 'Add', 'BUTTON', 6, 2, 'AUTH', '85,89', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (231, '角色管理-编辑', 'Edit', 'LINK', 6, 1, 'AUTH', '83,91', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (232, '角色管理-删除', 'Delete', 'LINK', 6, 2, 'AUTH', '84,90', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (233, '角色管理-列表', 'List', 'LIST', 6, 1, 'AUTH', '83', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (234, '组织架构-查询条件', 'Condition', 'CONDITION', 4, 1, 'AUTH', '92', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (235, '角色管理-查询条件', 'Condition', 'CONDITION', 6, 1, 'AUTH', '83', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (236, '考务规则-考务规则配置', 'Add', 'BUTTON', 8, 1, 'AUTH', '99, 100', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (237, '题卡规则-查询', 'Select', 'BUTTON', 9, 1, 'AUTH', '101', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (238, '题卡规则-添加', 'Add', 'BUTTON', 9, 2, 'AUTH', '103', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (239, '题卡规则-查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', '101', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (240, '题卡规则-列表', 'List', 'LIST', 9, 1, 'AUTH', '101', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (241, '通卡模版-查询', 'Select', 'BUTTON', 10, 1, 'AUTH', '105', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (242, '通卡模版-添加模版', 'Add', 'BUTTON', 10, 2, 'AUTH', '106', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (243, '用户管理-批量导入', 'Export', 'BUTTON', 5, 3, 'AUTH', '81', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (244, '通卡模版-预览', 'Preview', 'LINK', 10, 1, 'AUTH', '192', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (245, '通卡模版-编辑', 'Edit', 'LINK', 10, 2, 'AUTH', '106', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (246, '通卡模版-启用&禁用', 'Enable', 'LINK', 10, 3, 'AUTH', '107', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (247, '通卡模版-查询条件', 'Condition', 'CONDITION', 10, 1, 'AUTH', '105', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (248, '通卡模版-列表', 'List', 'LIST', 10, 1, 'AUTH', '105', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (249, '变量印品模版-查询', 'Select', 'BUTTON', 11, 1, 'AUTH', '123', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (250, '变量印品模版-添加模版', 'Add', 'BUTTON', 11, 2, 'AUTH', '124', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (251, '变量印品模版-预览', 'Preview', 'LINK', 11, 1, 'AUTH', '192', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (252, '变量印品模版-编辑', 'Edit', 'LINK', 11, 2, 'AUTH', '124', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (253, '变量印品模版-启用&禁用', 'Enable', 'LINK', 11, 3, 'AUTH', '125', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (254, '变量印品模版-查询条件', 'Condition', 'CONDITION', 11, 1, 'AUTH', '123', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (255, '变量印品模版-列表', 'List', 'LIST', 11, 1, 'AUTH', '123', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (256, '普通印品模版-查询', 'Select', 'BUTTON', 12, 1, 'AUTH', '126', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (257, '普通印品模版-添加模版', 'Add', 'BUTTON', 12, 2, 'AUTH', '127', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (258, '普通印品模版-预览', 'Preview', 'LINK', 12, 1, 'AUTH', '192', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (259, '普通印品模版-编辑', 'Edit', 'LINK', 12, 2, 'AUTH', '127', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (260, '普通印品模版-启用&禁用', 'Enable', 'LINK', 12, 3, 'AUTH', '128', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (261, '普通印品模版-查询条件', 'Condition', 'CONDITION', 12, 1, 'AUTH', '126', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (262, '普通印品模版-列表', 'List', 'LIST', 12, 1, 'AUTH', '126', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (263, '课程管理-查询', 'Select', 'BUTTON', 13, 1, 'AUTH', '96,462', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (265, '课程管理-模版下载', 'TempleteDownload', 'BUTTON', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (268, '课程管理-编辑', 'Edit', 'LINK', 13, 1, 'AUTH', '97', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (269, '课程管理-批量启用&禁用', 'Enable', 'LINK', 13, 2, 'AUTH', '368', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (270, '课程管理-查询条件', 'Condition', 'CONDITION', 13, 2, 'AUTH', '96', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (271, '课程管理-列表', 'List', 'LIST', 13, 1, 'AUTH', '96', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (272, '命题任务管理-查询', 'Select', 'BUTTON', 39, 1, 'AUTH', '63', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (273, '命题任务管理-批量新建命题任务', 'BatchAdd', 'BUTTON', 39, 2, 'AUTH', '67', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (274, '命题任务管理-新建命题任务', 'Add', 'BUTTON', 39, 3, 'AUTH', '66,169,441,442,445,482', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (275, '命题任务管理-预览', 'Preview', 'LINK', 39, 1, 'AUTH', '68', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (276, '命题任务管理-编辑', 'Edit', 'LINK', 39, 2, 'AUTH', '63', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (277, '命题任务管理-取消任务', 'Cancel', 'LINK', 39, 3, 'AUTH', '65', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (278, '命题任务管理-查询条件', 'Condition', 'CONDITION', 39, 1, 'AUTH', '63', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (279, '命题任务管理-列表', 'List', 'LIST', 39, 1, 'AUTH', '63', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (280, '入库申请-查询', 'Select', 'BUTTON', 40, 1, 'AUTH', '70', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (281, '入库申请-预览', 'Preview', 'LINK', 40, 1, 'AUTH', '168,477,568,417,420,421,441,442,640,641', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (282, '入库申请-撤销申请', 'End', 'LINK', 40, 2, 'AUTH', '72', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (283, '入库申请-编辑', 'Edit', 'LINK', 40, 3, 'AUTH', '71,167,174,424,477,417,420,421,441,442,640,641', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (284, '入库申请-查询条件', 'Condition', 'CONDITION', 40, 1, 'AUTH', '70', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (285, '入库申请-列表', 'List', 'LIST', 40, 1, 'AUTH', '70', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (288, '入库审核-未审核查询', 'NotReviewSelect', 'BUTTON', 41, 1, 'AUTH', '73', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (289, '入库审核-未审核导出审核样本', 'NotReviewExport', 'BUTTON', 41, 2, 'AUTH', '163', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (290, '入库审核-未审核批量不通过', 'NotReviewBatchNotPass', 'BUTTON', 41, 3, 'AUTH', '75', 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (291, '入库审核-未审核批量通过', 'NotReviewBatchPass', 'BUTTON', 41, 4, 'AUTH', '75', 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (292, '入库审核-未审核编辑', 'NotReviewEdit', 'LINK', 41, 1, 'AUTH', '74,417,420,421,441,442,477,417,640,641', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (293, '入库审核-未审核查询条件', 'NotReviewCondition', 'CONDITION', 41, 1, 'AUTH', '73', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (294, '入库审核-未审核列表', 'NotReviewList', 'LIST', 41, 1, 'AUTH', '73', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (295, '入库审核-已审核查询', 'ReviewSelect', 'BUTTON', 41, 1, 'AUTH', '166', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (296, '入库审核-已审核预览', 'ReviewPreview', 'LINK', 41, 1, 'AUTH', '74,477,417,420,421,441,442,640,641', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (297, '入库审核-已审核查询条件', 'ReviewCondition', 'CONDITION', 41, 1, 'AUTH', '166', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (298, '入库审核-已审核列表', 'ReviewList', 'LIST', 41, 1, 'AUTH', '166', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (299, '卷库查询-查询', 'Select', 'BUTTON', 42, 1, 'AUTH', '76', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (300, '卷库查询-批量下载试卷题卡', 'BatchDownload', 'BUTTON', 42, 2, 'AUTH', '191', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (301, '卷库查询-预览', 'Preview', 'LINK', 42, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (302, '卷库查询-启用&禁用', 'Enable', 'LINK', 42, 2, 'AUTH', '165', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (303, '卷库查询-下载', 'Download', 'LINK', 42, 3, 'AUTH', '164', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (304, '卷库查询-查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '76', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (305, '卷库查询-列表', 'List', 'LIST', 42, 1, 'AUTH', '76', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (306, '卷库查询-发布印刷任务', 'Publish', 'LINK', 42, 4, 'AUTH', '61,359,425,426,427,428,429,430,449', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (307, '入库申请-新建命题申请', 'AddTask', 'BUTTON', 40, 2, 'AUTH', '167,174,371,423,424,436,417,420,421,441,442,640,641', 0, 0, 0);
INSERT INTO `sys_privilege` VALUES (308, '卷库查询-编辑', 'Edit', 'LINK', 42, 4, 'AUTH', '189', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (309, '印刷计划管理-查询', 'Select', 'BUTTON', 43, 1, 'AUTH', '48,49', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (310, '印刷计划管理-新建印刷计划', 'Add', 'BUTTON', 43, 2, 'AUTH', '50,49', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (311, '印刷计划管理-预览', 'Preview', 'LINK', 43, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (312, '印刷计划管理-编辑', 'Edit', 'LINK', 43, 2, 'AUTH', '50', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (313, '印刷计划管理-删除', 'Delete', 'LINK', 43, 3, 'AUTH', '51', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (314, '印刷计划管理-查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '48', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (315, '印刷计划管理-列表', 'List', 'LIST', 43, 1, 'AUTH', '48,49', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (316, '考务数据导入-查询', 'Select', 'BUTTON', 44, 1, 'AUTH', '52,53', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (317, '考务数据导入-考务数据模版下载', 'TempleteDownload', 'BUTTON', 44, 2, 'AUTH', '157', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (318, '考务数据导入-导出查询结果', 'Export', 'BUTTON', 44, 3, 'AUTH', '158', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (319, '考务数据导入-导入', 'Import', 'BUTTON', 44, 4, 'AUTH', '54', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (320, '考务数据导入-预览', 'Preview', 'LINK', 44, 1, 'AUTH', '55', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (321, '考务数据导入-查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', '52', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (322, '考务数据导入-列表', 'List', 'LIST', 44, 1, 'AUTH', '52,53', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (323, '考务明细查询-查询', 'Select', 'BUTTON', 45, 1, 'AUTH', '159', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (324, '考务明细查询-预览', 'Preview', 'LINK', 45, 1, 'AUTH', '188', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (325, '考务明细查询-查询条件', 'Condition', 'CONDITION', 45, 1, 'AUTH', '159', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (326, '考务明细查询-列表', 'List', 'LIST', 45, 1, 'AUTH', '159', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (327, '关联试卷管理-查询', 'Select', 'BUTTON', 46, 1, 'AUTH', '56', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (328, '关联试卷管理-编辑', 'Edit', 'LINK', 46, 1, 'AUTH', '57,58', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (329, '关联试卷管理-查询条件', 'Condition', 'CONDITION', 46, 1, 'AUTH', '56', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (330, '关联试卷管理-列表', 'List', 'LIST', 46, 1, 'AUTH', '56', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (331, '印刷任务管理-查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '59,62', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (332, '印刷任务管理-下载', 'Download', 'LINK', 47, 1, 'AUTH', '161,162', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (333, '印刷任务管理-查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', '59', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (334, '印刷任务管理-列表', 'List', 'LIST', 47, 1, 'AUTH', '59', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (335, '印刷任务管理-撤回', 'End', 'LINK', 47, 2, 'AUTH', '61', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (336, '印刷任务管理-批量下载PDF', 'BatchDownload', 'BUTTON', 47, 2, 'AUTH', '336', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (337, '任务管理-查询', 'Select', 'BUTTON', 113, 1, 'AUTH', '114', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (338, '任务管理-导出日志', 'Export', 'LINK', 113, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (339, '任务管理-下载文件', 'Download', 'LINK', 113, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (340, '任务管理-查询条件', 'Condition', 'CONDITION', 113, 1, 'AUTH', '114', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (341, '任务管理-列表', 'List', 'LIST', 113, 1, 'AUTH', '114', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (342, '题卡规则-预览', 'Preview', 'LINK', 9, 1, 'AUTH', '102', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (343, '题卡规则-编辑', 'Edit', 'LINK', 9, 2, 'AUTH', '103', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (345, '题卡规则-启用&禁用', 'Enable', 'LINK', 9, 3, 'AUTH', '104', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (346, '题卡审核-未审核查询', 'NotReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (347, '题卡审核-未审核批量下载试卷文件', 'NotReviewBatchDownload', 'BUTTON', 172, 2, 'AUTH', '185', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (348, '题卡审核-未审核查询条件', 'NotReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (349, '题卡审核-未审核列表', 'NotReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (350, '题卡审核-未审核预览', 'NotReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (351, '题卡审核-未审核设计题卡', 'NotReviewDesign', 'LINK', 172, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (352, '题卡审核-已审核查询', 'ReviewSelect', 'BUTTON', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (353, '题卡审核-已审核查询条件', 'ReviewCondition', 'CONDITION', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (354, '题卡审核-已审核列表', 'ReviewList', 'LIST', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (355, '题卡审核-已审核预览', 'ReviewPreview', 'LINK', 172, 1, 'AUTH', '173', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (356, '字典管理', 'dictionary', 'MENU', 1, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (357, '校区管理', 'CampusManage', 'MENU', 356, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (358, '学生管理', 'StudentManage', 'MENU', 356, 6, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (359, '校区管理-查询', '/api/admin/basic/campus/query', 'URL', 357, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (360, '校区管理-新增/编辑', '/api/admin/basic/campus/save', 'URL', 357, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (361, '校区管理-删除（逻辑）', '/api/admin/basic/campus/delete', 'URL', 357, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (362, '学生基本信息管理-查询', '/api/admin/basic/student/query', 'URL', 358, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (363, '学生基本信息管理-新增/编辑', '/api/admin/basic/student/save', 'URL', 358, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (364, '学生基本信息管理-批量删除（逻辑）', '/api/admin/basic/student/delete_batch', 'URL', 358, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (365, '学生基本信息管理-批量导入（异步）', '/api/admin/basic/student/data_import', 'URL', 358, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (366, '课程管理-查询', '/api/admin/basic/course/query', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (367, '课程管理-新增/编辑', '/api/admin/basic/course/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (368, '课程管理-批量启用&禁用', '/api/admin/basic/course/enable', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (369, '课程基本信息管理-批量导入（异步）', '/api/admin/basic/course/data_import', 'URL', 13, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (370, '获取系统编码', '/api/admin/common/get_code', 'URL', 149, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (371, '根据类型查询机构', '/api/admin/sys/org/find_by_type', 'URL', 4, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (372, '校区管理-查询', 'Select', 'BUTTON', 357, 1, 'AUTH', '359', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (373, '校区管理-列表', 'List', 'LIST', 357, 1, 'AUTH', '359', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (374, '校区管理-新增', 'Add', 'BUTTON', 357, 1, 'AUTH', '360,370', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (375, '校区管理-编辑', 'Edit', 'LINK', 357, 1, 'AUTH', '360', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (376, '校区管理-删除', 'Delete', 'LINK', 357, 1, 'AUTH', '361', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (377, '学生管理-查询', 'Select', 'BUTTON', 358, 1, 'AUTH', '362,462', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (378, '学生管理-列表', 'List', 'LIST', 358, 1, 'AUTH', '362', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (379, '学生管理-新增', 'Add', 'BUTTON', 358, 1, 'AUTH', '359,363,462', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (380, '学生管理-编辑', 'Edit', 'LINK', 358, 1, 'AUTH', '359,363', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (381, '学生管理-删除', 'Delete', 'LINK', 358, 1, 'AUTH', '364', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (382, '学生管理-批量导入', 'Import', 'BUTTON', 358, 2, 'AUTH', '365', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (385, '课程管理-新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '367,462', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (388, '课程管理-批量导入', 'Import', 'BUTTON', 13, 2, 'AUTH', '369', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (389, '课程管理-机构列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (390, '流程管理', 'flow', 'MENU', 1, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (392, '印刷计划合并管理', 'PrintPlanPushManage', 'MENU', 3, 6, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (393, '印刷计划合并管理-查询', 'Select', 'BUTTON', 392, 1, 'AUTH', '49,431', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (394, '印刷计划合并管理-批量推送', 'BatchSend', 'BUTTON', 392, 2, 'AUTH', '434', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (395, '印刷计划合并管理-合并推送印刷计划', 'MergeSend', 'BUTTON', 392, 3, 'AUTH', '432,433', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (396, '印刷计划合并管理-预览', 'Preview', 'LINK', 392, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (397, '印刷计划合并管理-推送', 'Send', 'LINK', 392, 2, 'AUTH', '435', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (398, '印刷计划合并管理-查询条件', 'Condition', 'CONDITION', 392, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (399, '印刷计划合并管理-列表', 'List', 'LIST', 392, 1, 'AUTH', '431', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (400, '学生管理-查询条件', 'Condition', 'CONDITION', 358, 1, 'AUTH', '377', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (402, '流程管理', 'FlowManage', 'MENU', 390, 1, NULL, '402,403,404,405,406,407,408,409,410,411,414,415,416,417,418,419,420,421,422,441,442,567,638,639,640,641,643,644,184', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (403, '流程审批记录管理', 'ApproveRecordManage', 'MENU', 390, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (404, '流程审批记录管理-查询', 'Select', 'BUTTON', 403, 1, 'AUTH', '418', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (405, '流程审批记录管理-查询条件', 'Condition', 'CONDITION', 403, 1, 'AUTH', '418', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (406, '流程审批记录管理-列表', 'List', 'LIST', 403, 1, 'AUTH', '418', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (407, '流程管理-查询', 'Select', 'BUTTON', 402, 1, 'AUTH', '416', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (408, '流程管理-新建流程', 'Register', 'BUTTON', 402, 2, 'AUTH', '414', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (409, '流程管理-删除', 'Delete', 'LINK', 402, 2, 'AUTH', '415', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (410, '流程管理-查询条件', 'Condition', 'CONDITION', 402, 1, 'AUTH', '416', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (411, '流程管理-列表', 'List', 'LIST', 402, 1, 'AUTH', '416', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (412, '印刷计划管理-提交印刷', 'Submit', 'BUTTON', 43, 3, 'AUTH', '48', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (413, '任务管理-重新生成pdf', 'CreatePdf', 'LINK', 113, 3, 'AUTH', '190', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (414, '流程管理-保存和发布流程', '/api/admin/custom/flow/save', 'URL', 402, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (415, '流程管理-流程逻辑删除', '/api/admin/custom/flow/enable', 'URL', 402, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (416, '流程管理-流程列表', '/api/admin/custom/flow/list', 'URL', 402, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (417, '流程管理-审批流程', '/api/admin/custom/flow/task/approve', 'URL', 402, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (418, '流程管理-流程审批记录列表', '/api/admin/custom/flow/approve/list', 'URL', 402, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (419, '流程管理-流程审批记录逻辑删除', '/api/admin/custom/flow/approve/enable', 'URL', 402, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (420, '流程管理-流程终止', '/api/admin/custom/flow/end', 'URL', 402, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (421, '流程管理-获取所有流程节点', '/api/admin/custom/flow/task/all', 'URL', 402, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (422, '流程审批记录管理-删除', 'Delete', 'LINK', 403, 1, 'AUTH', '419', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (423, '入库申请-新建命题任务', '/api/admin/exam/task/save_task_apply', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (424, '入库申请-新建命题任务-切换题卡', '/api/admin/exam/task/switch_card', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (425, '卷库查询-发布印刷任务-查看列表', '/api/admin/exam/task/list_task_print', 'URL', 42, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (426, '卷库查询-发布印刷任务-班级列表', '/api/admin/exam/task/list_task_print_class', 'URL', 42, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (427, '卷库查询-发布印刷任务-印刷室列表', '/api/admin/exam/task/list_task_print_house', 'URL', 42, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (428, '卷库查询-发布印刷任务', '/api/admin/exam/task/save_task_print', 'URL', 42, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (429, '卷库查询-发布印刷任务-查看考生', '/api/admin/exam/task/list_task_print_student', 'URL', 42, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (430, '卷库查询-发布印刷任务-删除', '/api/admin/exam/task/remove_task_print', 'URL', 42, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (431, '印刷计划合并管理-计划查询', '/api/admin/exam/print_sync/list_sync', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (432, '印刷计划合并管理-合并推送-云阅卷考试ID列表', '/api/admin/exam/print_sync/list_relate_ids', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (433, '印刷计划合并管理-合并推送-多计划合并同步', '/api/admin/exam/print_sync/sync_data_merge', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (434, '印刷计划合并管理-批量同步', '/api/admin/exam/print_sync/sync_data_cloud_batch', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (435, '印刷计划合并管理-同步失败-单个手动同步', '/api/admin/exam/print_sync/sync_data_cloud', 'URL', 392, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (436, '入库申请-新建命题任务-取消/删除', '/api/admin/exam/task/remove_task_apply', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (437, '印刷计划管理-查询模板', 'List', 'LIST', 43, 1, 'AUTH', '49', 0, 1, 0);
INSERT INTO `sys_privilege` VALUES (438, '考务数据导入-汇总数据查询', 'List', 'LIST', 44, 2, 'AUTH', '53', 0, 1, 0);
INSERT INTO `sys_privilege` VALUES (439, '题卡标题查询', '/api/admin/exam/task/find_card_title', 'URL', 40, 7, 'AUTH', NULL, 0, 1, 0);
INSERT INTO `sys_privilege` VALUES (440, '题卡标题查询', 'List', 'LIST', 40, 1, 'AUTH', '439', 0, 1, 0);
INSERT INTO `sys_privilege` VALUES (441, '流程管理-根据流程类型获取流程节点', '/api/admin/custom/flow/get_flow_info_by_type', 'URL', 402, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (442, '流程管理-流程节点转他人审批', '/api/admin/custom/flow/task/approver/exchange', 'URL', 402, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (443, '主任审核', 'Director', 'MENU', 390, 1, NULL, '417,441,442,482', 0, 1, 0);
INSERT INTO `sys_privilege` VALUES (444, '院长审核', 'President', 'MENU', 390, 2, NULL, '417,441,442,482', 0, 1, 0);
INSERT INTO `sys_privilege` VALUES (446, '印刷任务管理-做废/恢复', '/api/admin/exam/print/task_normal', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (447, '印刷任务管理-做废/恢复', 'Normal', 'LINK', 47, 3, 'AUTH', '446', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (448, '命题任务管理-更改/指派命题老师', 'ASSIGN', 'LINK', 39, 4, 'AUTH', '64', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (449, '卷库查询-发布印刷任务-提交任务', '/api/admin/exam/task/submit_task_print', 'URL', 42, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (450, '班级管理', 'ClazzManage', 'MENU', 356, 5, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (451, '班级基本信息管理-查询', '/api/admin/basic/clazz/query', 'URL', 450, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (452, '班级基本信息管理-新增/编辑', '/api/admin/basic/clazz/save', 'URL', 450, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (453, '班级基本信息管理-批量删除（逻辑）', '/api/admin/basic/clazz/delete_batch', 'URL', 450, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (454, '班级基本信息管理-批量导入（异步）', '/api/admin/basic/clazz/data_import', 'URL', 450, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (455, '班级管理-查询', 'Select', 'BUTTON', 450, 1, 'AUTH', '451', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (456, '班级管理-列表', 'List', 'LIST', 450, 1, 'AUTH', '451', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (457, '班级管理-新增', 'Add', 'BUTTON', 450, 1, 'AUTH', '452', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (458, '班级管理-编辑', 'Edit', 'LINK', 450, 1, 'AUTH', '452', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (459, '班级管理-删除', 'Delete', 'LINK', 450, 1, 'AUTH', '453', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (460, '班级管理-批量导入', 'Import', 'BUTTON', 450, 2, 'AUTH', '454', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (461, '班级管理-查询条件', 'Condition', 'CONDITION', 450, 1, 'AUTH', '451', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (462, '班级数据源查询', '/api/admin/basic/clazz/datasource', 'URL', 450, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (463, '入库申请-新建命题任务', 'ExamTaskApplyManage', 'BUTTON', 40, 8, 'AUTH', '49,167,174,371,464,465,427,642,417,420,421,441,442,640,641,642', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (464, '新建命题申请', '/api/admin/exam/task/submit_task_apply', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (465, '新建命题申请-班级列表', '/api/admin/exam/task/list_task_apply_class', 'URL', 39, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (466, '命题任务-审批-获取试卷审批表数据', '/api/admin/exam/task/find_approve_form_json', 'URL', 39, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (467, '命题任务-审批-获取试卷审批表pdf', '/api/admin/exam/task/find_approve_form_pdf', 'URL', 39, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (468, '入库申请-下载审批表', 'Download', 'LINK', 40, 5, 'AUTH', '466,467', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (469, '命题计划统计', 'Statistics', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (470, '命题计划统计表', 'StatisticsManage', 'MENU', 469, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (471, '命题计划统计表-查询', 'Select', 'BUTTON', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (472, '命题计划统计表-模板下载', 'Download', 'BUTTON', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (473, '命题计划统计表-导入', 'Import', 'BUTTON', 470, 3, 'AUTH', '475', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (474, '命题计划统计表-列表', 'List', 'LIST', 470, 1, 'AUTH', '476', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (475, '命题计划统计-导入', '/api/admin/statistics/import', 'URL', 470, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (476, '命题计划统计-列表', '/api/admin/statistics/list', 'URL', 470, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (477, '命题任务-考试对象查询', '/api/admin/exam/task/find_exam_object', 'URL', 39, 12, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (478, '命题计划统计-刷新数据', 'Freshen', 'BUTTON', 470, 4, 'AUTH', '480', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (479, '命题计划统计-删除数据', 'Delete', 'BUTTON', 470, 5, 'AUTH', '481', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (480, '命题计划统计-刷新数据', '/api/admin/statistics/freshen', 'URL', 470, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (481, '命题计划统计-删除数据', '/api/admin/statistics/delete', 'URL', 470, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (483, '阅卷管理', 'stmms', 'MENU', NULL, 6, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (484, '评卷员', 'Marker', 'MENU', 483, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (485, '课组长', 'SubjectHeader', 'MENU', 483, 2, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (486, '阅卷管理', 'MarkTaskManage', 'MENU', 483, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (487, '成绩归档', 'ScoreArchive', 'MENU', 561, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (488, '评卷参数设置', 'UploadStructure', 'MENU', 486, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (489, '评卷参数设置-列表', '/api/admin/exam/structure/list', 'URL', 488, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (490, '评卷参数设置-上传', '/api/admin/exam/structure/upload', 'URL', 488, 2, 'AUTH', NULL, 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (491, '阅卷管理', 'MarkerLogin', 'MENU', 486, 1, NULL, '492,493,541', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (492, '评卷员登录', '/api/admin/exam/sso/marker_login', 'URL', 491, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (493, '科组长登录', '/api/admin/exam/sso/marker_leader_login', 'URL', 491, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (494, '评卷参数设置-列表', 'List', 'LIST', 488, 1, 'AUTH', '489', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (495, '评卷参数设置-上传试卷结构/标答', 'Upload', 'LINK', 488, 1, 'AUTH', '490,540', 0, 1, 1);
INSERT INTO `sys_privilege` VALUES (496, '评卷参数设置-查看试卷结构', 'Preview', 'LINK', 488, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (497, '评卷参数设置-查看标答', 'Preview', 'LINK', 488, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (498, '成绩归档-列表', 'List', 'LIST', 487, 1, 'AUTH', '542', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (499, '成绩归档-查询条件', 'Condition', 'CONDITION', 487, 1, 'AUTH', '542', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (500, '成绩归档-查询', 'Select', 'BUTTON', 487, 1, 'AUTH', '542', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (501, '成绩归档-数据同步', 'Sync', 'BUTTON', 487, 2, 'AUTH', '546', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (502, '成绩归档-成绩导出', 'Export', 'BUTTON', 487, 3, 'AUTH', '543', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (503, '成绩归档-一键下载', 'BatchDownload', 'BUTTON', 487, 4, 'AUTH', '545', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (504, '成绩归档-查看原卷', 'Preview', 'LINK', 487, 1, 'AUTH', '563', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (505, '成绩归档-下载', 'Download', 'LINK', 487, 2, 'AUTH', '544', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (506, '归档管理', 'ScoreArchiveManage', 'MENU', NULL, 7, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (507, '菜单管理', 'MenuManage', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (508, '菜单管理-自定义菜单', 'CustomMenu', 'MENU', 507, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (509, '用户管理-同步', 'SyncUser', 'BUTTON', 5, 1, 'AUTH', '532', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (510, '学期管理', 'SemesterManage', 'MENU', 356, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (511, '学期管理-新建学期', 'Add', 'BUTTON', 510, 1, 'AUTH', '534', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (512, '学期管理-编辑', 'Edit', 'LINK', 510, 1, 'AUTH', '534', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (513, '学期管理-删除', 'Delete', 'LINK', 510, 2, 'AUTH', '535', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (514, '学期管理-列表', 'List', 'LIST', 510, 1, 'AUTH', '533', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (515, '专业管理', 'MajorManage', 'MENU', 356, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (516, '专业管理-新增专业', 'Add', 'BUTTON', 515, 1, 'AUTH', '537', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (517, '专业管理-编辑', 'Edit', 'LINK', 515, 1, 'AUTH', '537', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (518, '专业管理-删除', 'Delete', 'LINK', 515, 2, 'AUTH', '538', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (519, '专业管理-列表', 'List', 'LIST', 515, 1, 'AUTH', '536', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (520, '专业管理-查询条件', 'Condition', 'CONDITION', 515, 1, 'AUTH', '536', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (521, '同步管理', 'SyncManage', 'MENU', 112, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (522, '专业管理-查询', 'Select', 'BUTTON', 515, 1, 'AUTH', '536', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (523, '同步管理-查询', 'Select', 'BUTTON', 521, 1, 'AUTH', '557', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (524, '同步管理-查询条件', 'Condition', 'CONDITION', 521, 1, 'AUTH', '557', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (525, '同步管理-列表', 'List', 'LIST', 521, 1, 'AUTH', '557', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (526, '同步管理-导出日志', 'Export', 'LINK', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (527, '菜单管理-查询自定义菜单权限', '/api/admin/menu/custom/list', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (528, '菜单管理-学校已绑定自定义菜单权限列表', '/api/admin/menu/custom/get_school_custom_privileges', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (529, '菜单管理-学校新增/修改自定义菜单权限', '/api/admin/menu/custom/save', 'URL', 508, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (530, '菜单管理-查询', 'Select', 'BUTTON', 508, 1, 'AUTH', '527,528', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (531, '菜单管理-保存', 'Save', 'BUTTON', 508, 2, 'AUTH', '529', 1, 0, 1);
INSERT INTO `sys_privilege` VALUES (532, '用户管理-同步', '/api/admin/sys/user/push', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (533, '学期管理-查询', '/api/admin/basic/semester/query', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (534, '学期管理-新增/编辑', '/api/admin/basic/semester/save', 'URL', 510, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (535, '学期管理-删除', '/api/admin/basic/semester/delete', 'URL', 510, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (536, '专业管理-查询', '/api/admin/basic/major/query', 'URL', 515, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (537, '专业管理-新增/编辑', '/api/admin/basic/major/save', 'URL', 515, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (538, '专业管理-删除）', '/api/admin/basic/major/delete', 'URL', 515, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (539, '共用接口-查询字典', '/api/admin/common/get_dictionary', 'URL', 149, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (540, '评卷参数设置-查看试卷结构', '/api/admin/exam/structure/preview_structure', 'URL', 488, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (541, '获取用户阅卷角色', '/api/admin/common/get_open_role', 'URL', 491, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (542, '成绩归档-查询接口', '/api/admin/sync/score/list', 'URL', 500, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (543, '成绩归档-成绩导出接口', '/api/admin/sync/score/export', 'URL', 502, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (544, '成绩归档-成绩动态轨迹图下载接口', '/api/admin/sync/score/download', 'URL', 505, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (545, '成绩归档-成绩动态轨迹图一键下载接口', '/api/admin/sync/score/batch_download', 'URL', 503, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (546, '成绩归档-成绩查询同步接口', '/api/admin/sync/score/sync', 'URL', 501, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (547, '阅卷试卷上传待办计数接口', '/api/admin/work/structure/task/ready_count', 'URL', 16, 7, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (548, '阅卷试卷上传待办接口', '/api/admin/work/structure/task/ready', 'URL', 16, 8, 'SYS', '490', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (549, '短信日志', 'SmsManage', 'MENU', 112, 3, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (550, '短信日志-查询', 'Select', 'BUTTON', 549, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (551, '短信日志-查询', '/api/admin/basic/message/list', 'URL', 550, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (552, '短信日志-查询短信类型', '/api/admin/basic/message/list_message_types', 'URL', 550, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (553, '短信日志-短信失败重发', '/api/admin/basic/message/resend', 'URL', 550, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (554, '短信日志-查询条件', 'Condition', 'CONDITION', 549, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (555, '短信日志-列表', 'List', 'LIST', 549, 1, 'AUTH', '551,552', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (556, '短信日志-短信失败重发', 'Send', 'LINK', 549, 1, 'AUTH', '553', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (557, '同步任务管理-查询', '/api/admin/data/sync/query', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (558, '学期管理-模糊查询', '/api/admin/basic/semester/list', 'URL', 510, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (559, '学期管理-设置当前使用学期', 'Edit', 'LINK', 510, 1, 'AUTH', '560', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (560, '学期管理-设置当前使用学期', '/api/admin/basic/semester/set_in_used', 'URL', 510, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (561, '归档管理', 'ScoreArchivePrimary', 'MENU', 506, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (562, '印刷任务管理-提交印刷', 'Submit', 'LINK', 47, 5, 'AUTH', '60', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (563, '成绩归档-查看原卷接口', '/api/admin/sync/score/preview', 'URL', 504, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (564, '考试试卷文件上传接口', '/api/admin/common/file/exam_paper_upload', 'URL', 144, 5, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (566, '审核-印刷员驳回后命题老师修改', '/api/admin/exam/task/apply_resave', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (567, '流程审批记录管理-查看', 'Preview', 'LINK', 403, 1, 'AUTH', '168,568,640,641', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (568, '入库申请-查询任务详情', '/api/admin/exam/task/exam_task_get_one', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (569, '学生基本信息管理-根据条件删除（物理）', '/api/admin/basic/student/delete_by_condition', 'URL', 358, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (570, '学生管理-根据条件删除', 'Delete', 'BUTTON', 358, 1, 'AUTH', '569', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (571, '班级管理-批量删除', 'Delete', 'BUTTON', 450, 1, 'AUTH', '453', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (572, '专业管理-批量删除', 'Delete', 'BUTTON', 515, 1, 'AUTH', '538', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (573, '课程管理-根据查询条件批量启用/禁用', '/api/admin/basic/course/enable_by_query', 'URL', 13, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (574, '课程管理-根据查询条件批量启用/禁用', 'Enable', 'BUTTON', 13, 5, 'AUTH', '573', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (575, '基础信息', 'basicInfo', 'MENU', 650, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (576, '教学课程管理', 'CourseSimpleManage', 'MENU', 575, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (577, '教学课程-查询', '/api/admin/teach/course/page', 'URL', 576, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (578, '教学课程-查找基础课程', '/api/admin/teach/course/find_basic_course', 'URL', 576, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (579, '教学课程-创建教学课程', '/api/admin/teach/course/create', 'URL', 576, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (580, '教学课程-批量删除', '/api/admin/teach/course/delete', 'URL', 576, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (581, '教学课程-列表', 'List', 'LIST', 576, 1, 'AUTH', '577', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (582, '教学课程-查询', 'Select', 'BUTTON', 576, 2, 'AUTH', '577', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (583, '教学课程-新增', 'Add', 'BUTTON', 576, 3, 'AUTH', '578,579', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (584, '教学课程-删除', 'Delete', 'LINK', 576, 4, 'AUTH', '580', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (585, '教学课程-班级管理', 'Link', 'LINK', 576, 5, 'AUTH', '587,588,589,590', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (586, '教学班级管理', 'ClazzSimpleManage', 'MENU', 575, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (587, '教学班级-查询', '/api/admin/teach/clazz/page', 'URL', 586, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (588, '教学班级-编辑教学班级', '/api/admin/teach/clazz/edit', 'URL', 586, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (589, '教学班级-批量删除', '/api/admin/teach/clazz/delete', 'URL', 586, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (590, '教学班级-查找教学课程', '/api/admin/teach/clazz/find_teach_course', 'URL', 586, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (591, '教学班级-列表', 'List', 'LIST', 586, 1, 'AUTH', '587', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (592, '教学班级-查询', 'Select', 'BUTTON', 586, 2, 'AUTH', '587', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (593, '教学班级-查询条件', 'Condition', 'CONDITION', 586, 3, 'AUTH', '590', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (594, '教学班级-新增', 'Add', 'BUTTON', 586, 4, 'AUTH', '588', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (595, '教学班级-重命名', 'Edit', 'LINK', 586, 5, 'AUTH', '588', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (596, '教学班级-删除', 'Delete', 'LINK', 586, 6, 'AUTH', '588', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (597, '教学班级-人员管理', 'Window', 'LINK', 586, 7, 'AUTH', '598,599,600,601,662', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (598, '教学学生-查询', '/api/admin/teach/student/page', 'URL', 597, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (599, '教学学生-创建教学学生', '/api/admin/teach/student/create', 'URL', 597, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (600, '教学学生-批量删除', '/api/admin/teach/student/delete', 'URL', 597, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (601, '教学学生-批量导入', '/api/admin/teach/student/import', 'URL', 597, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (602, '教学学生-列表', 'List', 'LIST', 597, 1, 'AUTH', '598', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (603, '教学学生-查询', 'Select', 'BUTTON', 597, 2, 'AUTH', '598', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (604, '教学学生-新增', 'Add', 'BUTTON', 597, 3, 'AUTH', '599', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (605, '教学学生-批量导入', 'Import', 'BUTTON', 597, 4, 'AUTH', '601', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (606, '教学学生-删除', 'Delete', 'LINK', 597, 5, 'AUTH', '600', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (607, '教学学生-批量删除', 'DeleteBatch', 'BUTTON', 597, 6, 'AUTH', '600', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (608, '教学课程-查询条件', 'Condition', 'CONDITION', 576, 6, 'AUTH', '577', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (609, '考试管理', 'ExamManage', 'MENU', 356, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (610, '考试管理-查询', 'Select', 'BUTTON', 609, 1, 'AUTH', '616', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (611, '考试管理-新增', 'Add', 'BUTTON', 609, 3, 'AUTH', '617', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (612, '考试管理-启用/禁用', 'Enable', 'LINK', 609, 4, 'AUTH', '618', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (613, '考试管理-删除', 'Delete', 'LINK', 609, 5, 'AUTH', '619', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (614, '考试管理-设置', 'Set', 'LINK', 609, 6, 'AUTH', '620,621,622,623,645,646', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (615, '考试管理-查询条件', 'Condition', 'CONDITION', 609, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (616, '考试管理-查询', '/api/admin/basic/exam/page', 'URL', 609, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (617, '考试管理-新增/修改', '/api/admin/basic/exam/save', 'URL', 609, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (618, '考试管理-启用/禁用', '/api/admin/basic/exam/enable', 'URL', 609, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (619, '考试管理-删除', '/api/admin/basic/exam/delete', 'URL', 609, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (620, '考试管理-设置', '/api/admin/basic/exam/set_in_userd', 'URL', 609, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (621, '印品配置管理-查询', '/api/admin/basic/print_config/page', 'URL', 614, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (622, '印品配置管理-新增/修改', '/api/admin/basic/print_config/save', 'URL', 614, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (623, '印品配置管理-启用/禁用', '/api/admin/basic/print_config/enable', 'URL', 614, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (624, '通卡管理', 'CardManage', 'MENU', 3, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (625, '题卡管理-查询', 'Select', 'BUTTON', 624, 1, 'AUTH', '627', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (626, '题卡管理-查询条件', 'Condition', 'CONDITION', 624, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (627, '题卡管理-查询', '/api/admin/exam/card/page', 'URL', 624, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (628, '考试管理-修改', 'Edit', 'LINK', 609, 7, 'AUTH', '617', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (629, '题卡管理-新增', 'Add', 'BUTTON', 624, 3, 'AUTH', '631', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (630, '题卡管理-修改', 'Edit', 'LINK', 624, 4, 'AUTH', '631', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (631, '题卡管理-新增/修改', '/api/admin/exam/card/save_generic', 'URL', 624, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (632, '考试管理-列表', 'List', 'LIST', 609, 7, 'AUTH', '616', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (633, '题卡管理-列表', 'List', 'LIST', 624, 5, 'AUTH', '627', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (634, '发送验证码（绑定手机号）', '/api/admin/common/get_verify_code_for_bind', 'URL', 130, 3, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (635, '题卡管理-删除', 'Delete', 'LINK', 624, 6, 'AUTH', '636', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (636, '题卡管理-删除', '/api/admin/exam/card/delete_generic', 'URL', 624, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (637, '考试管理-模糊查询', '/api/admin/basic/exam/query', 'URL', 609, 4, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (638, '流程管理-编辑', 'Edit', 'LINK', 402, 1, 'AUTH', '639', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (639, '流程管理-编辑流程', '/api/admin/custom/flow/edit', 'URL', 402, 14, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (640, '流程管理-获取当前流程节点信息', '/api/admin/custom/flow/task/info', 'URL', 402, 15, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (641, '流程管理-查看流程信息', '/api/admin/custom/flow/view', 'URL', 402, 16, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (642, '印品配置管理-根据考试ID和机构ID查询印品', '/api/admin/basic/print_config/get_by_exam_id_and_org_id', 'URL', 614, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (643, '流程管理-重命名', 'Rename', 'LINK', 402, 3, 'AUTH', '643', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (644, '流程管理-重命名自定义流程名称', '/api/admin/custom/flow/rename', 'URL', 402, 17, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (645, '印品配置管理-删除', '/api/admin/basic/print_config/delete', 'URL', 614, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (646, '印品配置管理-查询考试下设置印品的机构集合', '/api/admin/basic/print_config/list_org_id_by_exam_id', 'URL', 614, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (647, '角色管理-成员管理', 'Manager', 'LINK', 6, 3, 'AUTH', '648,649', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (648, '成员管理-新增/修改', '/api/admin/sys/role/group/save', 'URL', 647, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (649, '成员管理-查询', '/api/admin/sys/role/group/list', 'URL', 647, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (650, '基础信息', 'baseInfo', 'MENU', NULL, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (651, '电子交卷', 'electronic', 'MENU', NULL, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (652, '电子交卷', 'electronicSubmit', 'MENU', 651, 1, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (653, '下载管理', 'DownloadManage', 'MENU', 112, 4, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (654, '下载管理-查询', 'Select', 'BUTTON', 653, 1, 'AUTH', '659', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (655, '下载管理-查询条件', 'Condition', 'CONDITION', 653, 2, 'AUTH', '659', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (656, '下载管理-下载文件', 'Download', 'LINK', 653, 3, 'AUTH', '660', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (657, '下载管理-批量下载文件', 'Download', 'BUTTON', 653, 4, 'AUTH', '661', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (658, '下载管理-列表', 'List', 'LIST', 653, 5, 'AUTH', '659', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (659, '下载管理-查询', '/api/admin/data/download/page', 'URL', 653, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (660, '下载管理-下载文件', '/api/admin/data/download/download_one', 'URL', 653, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (661, '下载管理-批量下载文件', '/api/admin/data/download/download_batch', 'URL', 653, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (662, '教学学生-从课程班级批量添加学生', '/api/admin/teach/student/create_batch', 'URL', 597, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (663, '教学学生-从课程班级添加', 'SaveBatch', 'BUTTON', 597, 7, 'AUTH', '662', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (664, '同步任务管理-重新同步', 'Push', 'LINK', 521, 1, 'AUTH', '665', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (665, '同步任务管理-重新同步', '/api/admin/data/sync/resync', 'URL', 521, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (666, '评卷参数设置-上传试卷标答文件', 'Upload', 'LINK', 488, 4, 'AUTH', '667', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (667, '评卷参数设置-上传试卷标答文件', '/api/admin/exam/structure/upload_answer', 'URL', 488, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (668, '评卷参数设置-更新客观题答案', 'Upload', 'LINK', 488, 5, 'AUTH', '669', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (669, '评卷参数设置-更新客观题答案', '/api/admin/exam/structure/update_objective_answer', 'URL', 488, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (670, '评卷参数设置-评卷参数设置', 'Submit', 'LINK', 488, 6, 'AUTH', '671,672', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (671, '评卷参数设置-提交', '/api/admin/exam/structure/submit', 'URL', 488, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (672, '评卷参数设置-获取题卡图片', '/api/admin/exam/structure/find_jpg_file', 'URL', 488, 8, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (673, '评卷参数设置-查询条件', 'Condition', 'CONDITION', 488, 7, 'AUTH', '489', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (674, '评卷参数设置-查询按钮', 'Select', 'BUTTON', 488, 8, 'AUTH', '489', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (675, '授权配置', 'AuthManage', 'MENU', 1, 6, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (676, '授权配置-导出硬件信息按钮', 'Export', 'BUTTON', 675, 1, 'AUTH', '677,678,679', 1, 0, 0);
INSERT INTO `sys_privilege` VALUES (677, '导出硬件信息', '/api/admin/auth/export/device/info', 'URL', 676, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (678, '离线激活', '/api/admin/auth/offline/activation', 'URL', 676, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (679, '查询激活信息', '/api/admin/auth/select', 'URL', 676, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` VALUES (680, '教研分析', 'analyze', 'MENU', NULL, 7, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (681, '分析批次管理', 'AnalysisBatchManage', 'MENU', 680, 2, NULL, NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (682, '成绩分析批次-查询', '/api/admin/grade/batch/page', 'URL', 681, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (683, '成绩分析批次-新建', '/api/admin/grade/batch/save', 'URL', 681, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (684, '成绩分析批次-删除', '/api/admin/grade/batch/delete', 'URL', 681, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (685, '成绩分析批次-推送', '/api/admin/grade/batch/push', 'URL', 681, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (686, '成绩分析批次查询', 'Select', 'BUTTON', 681, 1, 'AUTH', '682', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (687, '成绩分析批次列表', 'List', 'LIST', 681, 2, 'AUTH', '682', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (688, '成绩分析批次新建', 'Add', 'BUTTON', 681, 3, 'AUTH', '683', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (689, '成绩分析批次批量删除', 'Delete', 'BUTTON', 681, 4, 'AUTH', '684', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (690, '成绩分析批次删除', 'Delete', 'LINK', 681, 5, 'AUTH', '684', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (691, '成绩分析批次推送', 'Push', 'LINK', 681, 6, 'AUTH', '685', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (692, '成绩分析批次添加分析课程', 'Window', 'LINK', 681, 7, 'AUTH', '693,694,695,696,704,705', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (693, '成绩分析批次课程-查询', '/api/admin/grade/batch/paper/page', 'URL', 692, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (694, '成绩分析批次课程-新建', '/api/admin/grade/batch/paper/save', 'URL', 692, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (695, '成绩分析批次课程-删除', '/api/admin/grade/batch/paper/delete', 'URL', 692, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (696, '成绩分析批次课程-查询可分析试卷', '/api/admin/grade/batch/paper/able', 'URL', 692, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (697, '成绩分析批次课程查询', 'Select', 'BUTTON', 692, 1, 'AUTH', '693', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (698, '成绩分析批次课程列表', 'List', 'LIST', 692, 2, 'AUTH', '693', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (699, '成绩分析批次课程添加分析课程', 'Add', 'BUTTON', 692, 3, 'AUTH', '694', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (700, '成绩分析批次课程批量删除', 'Delete', 'BUTTON', 692, 4, 'AUTH', '695', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (701, '成绩分析批次课程删除', 'Delete', 'BUTTON', 692, 5, 'AUTH', '695', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (702, '成绩分析批次课程可分析课程查询', 'Select', 'BUTTON', 692, 6, 'AUTH', '696', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (703, '成绩分析批次课程可分析课程列表', 'List', 'LIST', 692, 7, 'AUTH', '696', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (704, '成绩分析批次课程-推送', '/api/admin/grade/batch/paper/push', 'URL', 692, 5, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (705, '成绩分析批次课程-计算', '/api/admin/grade/batch/paper/calculate', 'URL', 692, 6, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (706, '成绩分析批次课程推送', 'Push', 'LINK', 692, 8, 'AUTH', '704', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (707, '成绩分析批次课程计算', 'Calculate', 'LINK', 692, 9, 'AUTH', '705', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (708, '数据初始化', 'DataInitManage', 'MENU', 680, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (709, '成绩分析课程数据初始化', '/api/admin/grade/paper/initialize', 'URL', 708, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (710, '成绩分析数据初始化查询', 'Select', 'BUTTON', 708, 1, 'AUTH', '709', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (711, '成绩分析数据初始化列表', 'List', 'LIST', 708, 2, 'AUTH', '709', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (712, '成绩分析基础配置', 'Window', 'LINK', 708, 3, 'AUTH', '717,718,719,720,726,727,728,729,735,736,737,738,755', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (717, '成绩分析维度-查询', '/api/admin/grade/paper/dimension/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (718, '成绩分析维度-新建维度', '/api/admin/grade/paper/dimension/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (719, '成绩分析维度-模板导出', '/api/admin/grade/paper/dimension/export', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (720, '成绩分析维度-导入', '/api/admin/grade/paper/dimension/import', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (721, '成绩分析维度查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '717', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (722, '成绩分析维度列表', 'List', 'LIST', 712, 2, 'AUTH', '717', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (723, '成绩分析维度新建', 'Add', 'BUTTON', 712, 3, 'AUTH', '718', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (724, '成绩分析维度导入', 'Import', 'BUTTON', 712, 4, 'AUTH', '720', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (725, '成绩分析维度模板导出', 'Export', 'BUTTON', 712, 5, 'AUTH', '719', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (726, '成绩分析试卷结构-查询', '/api/admin/grade/paper/struct/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (727, '成绩分析试卷结构-新建试卷结构', '/api/admin/grade/paper/struct/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (728, '成绩分析试卷结构-导入', '/api/admin/grade/paper/struct/import', 'URL', 712, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (729, '成绩分析试卷结构-模板导出', '/api/admin/grade/paper/struct/export', 'URL', 712, 4, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (730, '成绩分析试卷结构查询', 'Select', 'BUTTON', 712, 1, 'AUTH', '726', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (731, '成绩分析试卷结构列表', 'List', 'LIST', 712, 2, 'AUTH', '726', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (732, '成绩分析试卷结构导入', 'Import', 'BUTTON', 712, 3, 'AUTH', '728', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (734, '成绩分析试卷结构模板导出', 'Export', 'BUTTON', 712, 4, 'AUTH', '729', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (735, '成绩分析课程-查询定义', '/api/admin/grade/paper/paper_define', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (736, '成绩分析课程-新建定义', '/api/admin/grade/paper/save_define', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (737, '成绩分析模块评价-查询', '/api/admin/grade/module/evaluation/list', 'URL', 712, 1, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (738, '成绩分析模块评价-新建模块评价', '/api/admin/grade/module/evaluation/save', 'URL', 712, 2, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (739, '查看报告', 'AnalysisReportView', 'MENU', 680, 3, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (740, '教研分析单点登录', '/api/admin/exam/sso/analysis_login', 'URL', 149, 1, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (741, '成绩分析批次-考务数据导出', 'Export', 'LINK', 681, 8, 'AUTH', '743', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (742, '成绩分析批次-考务数据导入', 'Import', 'LINK', 681, 9, 'AUTH', '744', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (743, '成绩分析批次-考务数据导出', '/api/admin/grade/batch/download', 'URL', 681, 10, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (744, '成绩分析批次-考务数据导入', '/api/admin/grade/batch/upload', 'URL', 681, 11, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (745, '教研待办接口', '/api/admin/work/grade/task/ready', 'URL', 16, 9, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (746, '教研待办计数接口', '/api/admin/work/grade/task/ready_count', 'URL', 16, 10, 'SYS', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (747, '成绩分析批次-开始计算', 'StartCalc', 'LINK', 681, 12, 'AUTH', '748', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (748, '成绩分析批次-开始计算', '/api/admin/grade/batch/start_calc', 'URL', 681, 13, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (749, '成绩分析课程数据初始化查询条件', 'Condition', 'CONDITION', 708, 4, 'AUTH', '709', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (750, '成绩分析批次管理查询条件', 'Condition', 'CONDITION', 681, 14, 'AUTH', '682', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (751, '成绩分析批次课程发布', 'Publish', 'LINK', 681, 6, 'AUTH', '752', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (752, '成绩分析批次课程发布', '/api/admin/grade/batch/paper/publish', 'URL', 681, 7, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (753, '评卷参数设置-绑定科组长', 'Bind', 'LINK', 488, 8, 'AUTH', '754', 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (754, '评卷参数设置-绑定科组长', '/api/admin/exam/structure/bind_mark_leader', 'URL', 488, 9, 'AUTH', NULL, 1, 1, 1);
INSERT INTO `sys_privilege` VALUES (755, '成绩分析试卷结构-更新试卷机构', '/api/admin/grade/paper/struct/change_paper_structure', 'URL', 712, 5, 'AUTH', NULL, 1, 1, 1);

INSERT INTO `sys_user` VALUES (1, NULL, 'sysadmin', 'sysadmin', NULL, 'MTIzNDU2', NULL, NULL, 1, 1, 1629353956868, NULL, NULL, NULL, NULL, 1629353956881, NULL, NULL);

INSERT INTO `sys_user_role` VALUES (1, 1, 1, NULL, 1);

INSERT INTO `sys_config` VALUES (1, NULL, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (2, NULL, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (3, NULL, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (4, NULL, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (5, NULL, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (6, NULL, NULL, 'sys.sync.enable', '是否开启数据同步云阅卷', 'true', 'true-启用，false-禁用', 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (7, NULL, NULL, 'sys.txt.charset', 'txt文件编码', 'gb2312', NULL, 1, NULL, NULL, NULL);

INSERT INTO `sys_role` VALUES (1, NULL, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, NULL);
INSERT INTO `sys_role` VALUES (2, NULL, NULL, '教务处老师', 1, 'OFFICE_TEACHER', NULL, NULL, NULL, NULL, 1, 'ANALYSIS');
INSERT INTO `sys_role` VALUES (3, NULL, NULL, '学院院长', 1, 'PRESIDENT', NULL, NULL, NULL, NULL, 1, 'ANALYSIS');
INSERT INTO `sys_role` VALUES (4, NULL, NULL, '任课老师', 1, 'TEACHER', NULL, NULL, NULL, NULL, 1, 'ANALYSIS');

INSERT INTO `sys_role_privilege` VALUES (1, 1, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (2, 1, 2, 1);
INSERT INTO `sys_role_privilege` VALUES (3, 1, 4, 1);
INSERT INTO `sys_role_privilege` VALUES (4, 1, 5, 1);
INSERT INTO `sys_role_privilege` VALUES (5, 1, 6, 1);
INSERT INTO `sys_role_privilege` VALUES (6, 1, 92, 1);
INSERT INTO `sys_role_privilege` VALUES (7, 1, 93, 1);
INSERT INTO `sys_role_privilege` VALUES (8, 1, 94, 1);
INSERT INTO `sys_role_privilege` VALUES (9, 1, 95, 1);
INSERT INTO `sys_role_privilege` VALUES (10, 1, 224, 1);
INSERT INTO `sys_role_privilege` VALUES (11, 1, 225, 1);
INSERT INTO `sys_role_privilege` VALUES (12, 1, 226, 1);
INSERT INTO `sys_role_privilege` VALUES (13, 1, 227, 1);
INSERT INTO `sys_role_privilege` VALUES (14, 1, 228, 1);
INSERT INTO `sys_role_privilege` VALUES (15, 1, 234, 1);
INSERT INTO `sys_role_privilege` VALUES (16, 1, 77, 1);
INSERT INTO `sys_role_privilege` VALUES (17, 1, 78, 1);
INSERT INTO `sys_role_privilege` VALUES (18, 1, 79, 1);
INSERT INTO `sys_role_privilege` VALUES (19, 1, 80, 1);
INSERT INTO `sys_role_privilege` VALUES (20, 1, 81, 1);
INSERT INTO `sys_role_privilege` VALUES (21, 1, 82, 1);
INSERT INTO `sys_role_privilege` VALUES (22, 1, 87, 1);
INSERT INTO `sys_role_privilege` VALUES (23, 1, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (24, 1, 217, 1);
INSERT INTO `sys_role_privilege` VALUES (25, 1, 218, 1);
INSERT INTO `sys_role_privilege` VALUES (26, 1, 219, 1);
INSERT INTO `sys_role_privilege` VALUES (27, 1, 220, 1);
INSERT INTO `sys_role_privilege` VALUES (28, 1, 221, 1);
INSERT INTO `sys_role_privilege` VALUES (29, 1, 222, 1);
INSERT INTO `sys_role_privilege` VALUES (30, 1, 223, 1);
INSERT INTO `sys_role_privilege` VALUES (31, 1, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (32, 1, 83, 1);
INSERT INTO `sys_role_privilege` VALUES (33, 1, 84, 1);
INSERT INTO `sys_role_privilege` VALUES (34, 1, 85, 1);
INSERT INTO `sys_role_privilege` VALUES (35, 1, 86, 1);
INSERT INTO `sys_role_privilege` VALUES (36, 1, 91, 1);
INSERT INTO `sys_role_privilege` VALUES (37, 1, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (38, 1, 229, 1);
INSERT INTO `sys_role_privilege` VALUES (39, 1, 230, 1);
INSERT INTO `sys_role_privilege` VALUES (40, 1, 231, 1);
INSERT INTO `sys_role_privilege` VALUES (41, 1, 232, 1);
INSERT INTO `sys_role_privilege` VALUES (42, 1, 233, 1);
INSERT INTO `sys_role_privilege` VALUES (43, 1, 235, 1);
INSERT INTO `sys_role_privilege` VALUES (44, 1, 390, 1);
INSERT INTO `sys_role_privilege` VALUES (45, 1, 402, 1);
INSERT INTO `sys_role_privilege` VALUES (46, 1, 407, 1);
INSERT INTO `sys_role_privilege` VALUES (47, 1, 408, 1);
INSERT INTO `sys_role_privilege` VALUES (48, 1, 409, 1);
INSERT INTO `sys_role_privilege` VALUES (49, 1, 410, 1);
INSERT INTO `sys_role_privilege` VALUES (50, 1, 411, 1);
INSERT INTO `sys_role_privilege` VALUES (53, 1, 527, 1);
INSERT INTO `sys_role_privilege` VALUES (54, 1, 528, 1);
INSERT INTO `sys_role_privilege` VALUES (55, 1, 529, 1);
INSERT INTO `sys_role_privilege` VALUES (56, 1, 638, 1);
INSERT INTO `sys_role_privilege` VALUES (57, 1, 643, 1);
INSERT INTO `sys_role_privilege` VALUES (58, 1, 675, 1);
INSERT INTO `sys_role_privilege` VALUES (59, 1, 676, 1);
INSERT INTO `sys_role_privilege` VALUES (60, 1, 677, 1);
INSERT INTO `sys_role_privilege` VALUES (61, 1, 678, 1);
INSERT INTO `sys_role_privilege` VALUES (62, 1, 679, 1);
INSERT INTO `sys_role_privilege` VALUES (63, 2, 680, 1);
INSERT INTO `sys_role_privilege` VALUES (64, 2, 739, 1);
INSERT INTO `sys_role_privilege` VALUES (65, 3, 680, 1);
INSERT INTO `sys_role_privilege` VALUES (66, 3, 739, 1);
INSERT INTO `sys_role_privilege` VALUES (67, 4, 680, 1);
INSERT INTO `sys_role_privilege` VALUES (68, 4, 739, 1);


-- ----------------------------
-- Function structure for getRootChildId
-- ----------------------------
DROP FUNCTION IF EXISTS `getRootChildId`;
CREATE FUNCTION `getRootChildId`(rootId bigint) RETURNS varchar(1000) CHARSET utf8
    DETERMINISTIC
BEGIN
    DECLARE ptemp VARCHAR(1000);
    DECLARE ctemp VARCHAR(1000);
    SET ptemp = '#';
    SET ctemp = cast(rootId as CHAR);
    WHILE ctemp IS NOT NULL DO
      SET ptemp = concat(ptemp, ',', ctemp);
SELECT group_concat(id) INTO ctemp FROM sys_org
WHERE FIND_IN_SET(parent_id, ctemp) > 0;
END WHILE;
RETURN ptemp;
END